"""
    TranSPHIRE is supposed to help with the cryo-EM data collection
    Copyright (C) 2017 Markus Stabrin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import numpy as np
import json
import datetime
import sys
import os
import re
import copy
import shutil
import pexpect as pe
from PyQt5.QtWidgets import (
    QMainWindow,
    QHBoxLayout,
    QVBoxLayout,
    QWidget,
    QFileDialog,
    )
from PyQt5.QtCore import QThread, pyqtSlot, QCoreApplication, QTimer, pyqtSignal

from . import transphire_utils as tu
from . import transphire_import as ti

class MainWindow(QMainWindow):
    """
    MainWindow widget.
    Connects GUI and worker threads.

    Inherits from:
    QMainWindow

    Buttons:
    None

    Signals:
    None
    """
    sig_reset = pyqtSignal(str, object)

    def __init__(
            self, content_raw, content_gui, content_pipeline, settings_folder,
            mount_directory, template_name, version, n_feedbacks, parent=None
            ):
        """
        Setup the layout for the widget

        Arguments:
        content_gui - Content used to create the GUI outfit.
        content_pipeline - Content used to start processing threads.
        settings_folder - Name of the folder containing settings.
        mount_directory - Name of the folder containing mount points.
        version - Version of TranSPHIRE.
        parent - Parent widget (default None)

        Return:
        None
        """
        super(MainWindow, self).__init__(parent)

        # Ask for sudo password if needed
        self.password = ''
        need_sudo_password = False
        for content in content_gui:
            if content['name'] == 'Mount':
                for entry in content['content_mount']:
                    for widget in entry:
                        for key in widget:
                            if key == 'Need sudo for copy?':
                                if widget[key][0] == 'True':
                                    need_sudo_password = True
                                else:
                                    pass
                            elif key == 'Need sudo for mount?':
                                if widget[key][0] == 'True':
                                    need_sudo_password = True
                                else:
                                    pass
                            else:
                                pass
            else:
                pass

        if need_sudo_password:
            while True:
                dialog = InputBox(is_password=True, parent=self)
                dialog.setText('Sudo password', 'Please provide the sudo password for mount/copy!')
                dialog.exec_()
                if not dialog.result():
                    QCoreApplication.instance().quit()
                    sys.exit()
                else:
                    self.password = dialog.getText()
                    child = pe.spawnu('sudo -S -k ls')
                    child.sendline(self.password)
                    try:
                        idx = child.expect(
                            [
                                pe.EOF,
                                'sudo: 1 incorrect password attempt',
                                'Sorry, try again'
                                ],
                            timeout=10
                            )
                    except pe.exceptions.TIMEOUT:
                        tu.message('Wrong sudo password!')
                    else:
                        if idx == 0:
                            break
                        else:
                            print('Wrong sudo password!')
                            tu.message('Wrong sudo password!')
        else:
            pass

        # Start TranSPHIRE from HOME directory.
        self.path = os.getcwd()

        # Window title
        self.setWindowTitle(
            '{0} v{1} - {2} - {3}'.format(
                'TranSPHIRE',
                version,
                self.path,
                os.uname()[1]
                )
            )

        # Initiate contents
        self.central_widget_raw = None
        self.central_widget = None
        self.content = None
        self.content_raw = content_raw
        self.content_pipeline=content_pipeline
        self.layout = None
        self.n_feedbacks = n_feedbacks
        function_dict = tu.get_function_dict()
        self.default_types = ['mount', 'process', 'plotting']
        self.types = self.default_types[:]
        for value in function_dict.values():
            if value['typ'] is not None and value['typ'] not in self.types:
                #self.types.append(value['typ'])
                for index in range(self.n_feedbacks+1):
                    self.types.append('{0}_feedback_{1}'.format(value['typ'], index))

        # Settings folder
        self.settings_folder = settings_folder
        self.project_settings_folder = os.path.join(
            settings_folder,
            'PROJECT_SETTINGS'
            )
        try:
            tu.mkdir_p(self.project_settings_folder)
        except PermissionError:
            print('Settings directory is not writable.')
        self.mount_directory = mount_directory
        self.template_name = template_name
        self.temp_save = '{0}/temp_save_{1}'.format(
            self.settings_folder,
            os.uname()[1].replace(' ', '_')
            )

        # Threads
        self.mount_calculation_ssh = None
        self.mount_calculation_get = None
        self.mount_calculation_df = None
        self.timers = {}
        self.workers = {}
        self.threads = {}
        for entry in self.types:
            self.workers[entry] = None
            self.threads[entry] = None
            self.timers[entry] = None
        self.mount_thread_list = None

        self.sig_reset.connect(self.reset_gui)
        self.sig_reset.emit(template_name, None)

    def start_threads(self, content_pipeline):
        """
        Start threads used in TranSPHIRE.

        Arguments:
        content_pipeline - Content used to start processing threads.

        Return:
        None
        """
        # Stop threads if already started.
        for entry in self.types:
            if self.workers[entry] is not None:
                self.workers[entry].setParent(None)

            if self.threads[entry] is not None:
                self.threads[entry].quit()
                self.threads[entry].wait()
                self.threads[entry].setParent(None)

            if self.timers[entry] is not None:
                self.timers[entry].setParent(None)

        if self.mount_thread_list is not None:
            for setting in self.content['Mount'].get_settings():
                for key in setting:
                    thread = self.mount_thread_list[key]['thread']
                    calculator = self.mount_thread_list[key]['object']
                    calculator.kill_thread = True
                    thread.quit()
                    thread.wait()

        for entry in self.types:
            # Create objects used in threads
            if entry == 'mount':
                self.workers[entry] = MountWorker(
                    password=self.password,
                    settings_folder=self.settings_folder,
                    mount_directory=self.mount_directory
                    )
            elif entry == 'process':
                self.workers[entry] = ProcessWorker(
                    password=self.password,
                    content_process=content_pipeline,
                    mount_directory=self.mount_directory
                    )
            elif entry == 'plotting':
                self.workers[entry] = PlotWorker()
            else:
                self.workers[entry] = None

            if entry in self.default_types:
                # Create threads
                self.threads[entry] = QThread(self)
                # Start threads
                self.threads[entry].start()

                # Start objects in threads
                self.workers[entry].moveToThread(self.threads[entry])
            else:
                self.threads[entry] = None

    @pyqtSlot(str, object)
    def reset_gui(self, template_name, load_file):
        """
        Reset the content of the mainwindow.

        Arguments:
        template_name - Name of the template to load
        load_file - Settings file (default None).

        Return:
        None
        """
        self.template_name = template_name
        content_pipeline = self.content_pipeline
        content_gui = tu.get_content_gui(
            content=self.content_raw,
            template_name=template_name,
            n_feedbacks=self.n_feedbacks,
            )
        # Fill MainWindow
        self.set_central_widget()
        self.set_layout_structure()
        self.start_threads(content_pipeline=content_pipeline)
        postprocess_content = self.fill_content(content_gui=content_gui)
        self.postprocess_content(postprocess_content)

        for content in content_gui:
            if content['name'] == 'Status':
                for entry in content['content']:
                    for widget in entry:
                        for key in widget:
                            if key == 'Project name pattern':
                                self.project_name_pattern = widget[key][0]
                            elif key == 'Project name pattern example':
                                self.project_name_pattern_example = widget[key][0]
                            else:
                                pass
            else:
                pass

        # Load settings saved in load_file
        if load_file is not None and load_file:
            self.load(file_name=load_file)
            os.remove(load_file)
        elif os.path.exists('{0}.txt'.format(self.temp_save)) and load_file is None:
            # Result is True if answer is Yes
            result = tu.question(
                head='Restore default values.',
                text='Restore default values?',
                )
            if not result:
                self.load(file_name=self.temp_save)
            else:
                pass
        else:
            pass
        self.workers['mount'].sig_load_save.emit()

    def save_temp_settings(self):
        """
        Save the status of the GUI in a temp file.

        Arguments:
        None

        Return:
        True, if saving was succesful.
        """
        if os.path.exists('{0}.txt'.format(self.temp_save)):
            os.remove('{0}.txt'.format(self.temp_save))
        else:
            pass
        value = self.save(file_name=self.temp_save, temp=True, interactive=True)
        return value

    def postprocess_content(self, error_list):
        """
        Do postprocessing of creating GUI content, like connecting signals.

        Arguments:
        error_list - List of errors that occured.

        Return:
        True, if saving was succesful.
        """
        for entry in error_list:
            tu.message(entry)

        self.workers['process'].sig_finished.connect(self._finished)
        self.workers['process'].signal_plot.connect(self.workers['plotting'].set_settings)
        self.workers['plotting'].sig_set_visual.connect(self.set_visualisation)

        self.timers['plotting'] = QTimer(self)
        self.timers['plotting'].setInterval(20000)
        self.timers['plotting'].setSingleShot(True)
        self.timers['plotting'].timeout.connect(self.workers['plotting'].sig_calculate.emit)
        self.workers['plotting'].sig_new_round.connect(self.new_round_plot)

        self.mount_thread_list = {}
        for key in self.content['Mount'].content:
            thread = QThread(self)
            thread.start()
            mount_calculator = MountCalculator(name=key)
            mount_calculator.moveToThread(thread)
            self.mount_thread_list[key] = {
                'thread': thread,
                'object': mount_calculator
                }
            mount_calculator.sig_finished.connect(
                self.content['Status'].refresh_quota
                )
            mount_calculator.sig_finished.connect(
                self.abort_finished
                )
            self.workers['mount'].sig_calculate_ssh_quota.connect(
                mount_calculator.calculate_ssh_quota
                )
            self.workers['mount'].sig_calculate_df_quota.connect(
                mount_calculator.calculate_df_quota
                )
            self.workers['mount'].sig_calculate_get_quota.connect(
                mount_calculator.calculate_get_quota
                )
        self.content['Mount'].set_threadlist(thread_list=self.mount_thread_list)

    @pyqtSlot()
    def set_visualisation(self):
        self.content['TAB1'].setCurrentIndex(0)
        self.content['TAB1'].setCurrentIndex(3)

    @pyqtSlot()
    def new_round_plot(self):
        self.timers['plotting'].start()

    def abort_finished(self, *args, **kwargs):
        """
        Set the mount worker abort variable to True.

        Arguments:
        None

        Return:
        None
        """
        self.workers['mount'].abort_finished = True

    def set_central_widget(self):
        """
        Reset the central widget of the MainWindow.

        Arguments:
        None

        Return:
        None
        """
        if self.central_widget_raw is not None:
            self.central_widget_raw.setParent(None)
        else:
            pass
        if self.central_widget is not None:
            self.central_widget.setParent(None)
        else:
            pass

        self.central_widget_raw = QWidget(self)
        self.central_widget_raw.setObjectName('central_raw')
        self.setCentralWidget(self.central_widget_raw)
        layout = QVBoxLayout(self.central_widget_raw)
        self.central_widget = QWidget(self.central_widget_raw)
        self.central_widget.setObjectName('central')
        layout.addWidget(self.central_widget)

    def set_layout_structure(self):
        """
        Setup the layout structure for the central widget.

        Arguments:
        None

        Return:
        None
        """
        # Layout dictionary
        widget_layout_h2 = QWidget(self)
        widget_layout_h3 = QWidget(self)
        widget_layout_h4 = QWidget(self)
        widget_layout_v1 = QWidget(self)
        widget_layout_v1_a = QWidget(self)
        widget_layout_v2 = QWidget(self)

        self.layout = {}
        self.layout['h1'] = QHBoxLayout(self.central_widget)
        self.layout['h2'] = QHBoxLayout(widget_layout_h2)
        self.layout['h3'] = QHBoxLayout(widget_layout_h3)
        self.layout['h4'] = QHBoxLayout(widget_layout_h4)
        self.layout['v1'] = QVBoxLayout(widget_layout_v1)
        self.layout['v1_a'] = QVBoxLayout(widget_layout_v1_a)
        self.layout['v2'] = QVBoxLayout(widget_layout_v2)

        self.layout['h2'].setContentsMargins(0, 0, 0, 0)
        self.layout['h3'].setContentsMargins(0, 0, 0, 0)
        self.layout['h4'].setContentsMargins(0, 0, 0, 0)
        self.layout['v1'].setContentsMargins(0, 0, 0, 0)
        self.layout['v1_a'].setContentsMargins(0, 0, 0, 0)
        self.layout['v2'].setContentsMargins(0, 0, 0, 0)


        # Layout architecture
        self.layout['h1'].addWidget(widget_layout_v1, stretch=1)
        self.layout['h1'].addWidget(
            Separator(
                typ='vertical',
                color='black',
                left=widget_layout_v1,
                right=widget_layout_v2,
                ),
            stretch=0
            )
        self.layout['h1'].addWidget(widget_layout_v2, stretch=0)

        self.layout['v1_a'].addWidget(widget_layout_h2, stretch=0)
        self.layout['v1_a'].addWidget(
            Separator(
                typ='horizontal',
                color='grey',
                up=widget_layout_h2,
                down=widget_layout_h3,
                ),
            stretch=0
            )
        self.layout['v1_a'].addWidget(widget_layout_h3, stretch=0)

        self.layout['v1'].addWidget(widget_layout_v1_a, stretch=0)
        self.layout['v1'].addWidget(
            Separator(
                typ='horizontal',
                color='grey',
                up=widget_layout_v1_a,
                down=widget_layout_h4,
                ),
            stretch=0
            )
        self.layout['v1'].addWidget(widget_layout_h4, stretch=1)

    def fill_content(self, content_gui):
        """
        Fill the layouts of the central widget.

        Arguments:
        content_gui - Content used to create the GUI outfit.

        Return:
        List of errors that occured.
        """
        self.content = {}
        for entry in content_gui:
            if entry['name'] == 'Path':
                exclude_set = tu.get_exclude_set_path(content=entry['content'])
                tu.reduce_path_widget(exclude_set=exclude_set, content=entry['content'])
        for entry in content_gui:
            if entry['name'] == 'Copy':
                tu.reduce_copy_entries(exclude_set=exclude_set, content=entry['content'])
        error_list = []
        tab_list = []
        global_dict = {}
        for entry in content_gui:
            key = entry['name']

            if key in exclude_set:
                continue
            elif entry['layout'] in exclude_set:
                continue
            elif key == 'Stretch':
                layout = entry['layout']
                self.layout[layout].addStretch(1)
                continue
            elif key == 'Separator':
                layout = entry['layout']
                separator = entry['separator']
                self.layout[layout].addWidget(separator)
                continue
            else:
                pass

            layout = entry['layout']
            plot_labels = ''
            plot_name = ''
            try:
                plot_name = layout.replace('Plot ', '')
                if 'feedback' in plot_name:
                    plot_name_labels = plot_name[:-len(' feedback 1')]
                else:
                    plot_name_labels = plot_name
                plot_labels = ti.get_dtype_dict()[
                    tu.get_function_dict()[plot_name_labels]['typ']
                    ]
            except KeyError:
                pass

            # Create widget
            self.content[key] = entry['widget'](
                mount_worker=self.workers['mount'],
                process_worker=self.workers['process'],
                plot_worker=self.workers['plotting'],
                settings_folder=self.settings_folder,
                template_name=self.template_name,
                plot_labels=plot_labels,
                plot_name=plot_name,
                parent=self,
                global_dict=global_dict,
                **entry
                )

            if isinstance(self.content[key], TabDocker):
                tab_list.append(key)
            else:
                pass

            if layout in tab_list:
                self.content[layout].add_tab(self.content[key], key)
                self.content[key].setObjectName('tabbed')
            else:
                self.layout[layout].addWidget(
                    self.content[key]
                    )

            if key.startswith('Cinderella'):
                self.content['Retrain'].sig_new_config.connect(
                    self.content[key].set_new_model
                    )

            if key == 'Copy':
                self.content[key].sig_adjust_tab.connect(self.hide_tab)

            if key == 'Button':
                self.content[key].sig_load.connect(self.load)
                self.content[key].sig_save.connect(self.save)
                self.content[key].sig_start.connect(self.start)
                self.content[key].sig_stop.connect(self.stop_dialog)
                self.content[key].sig_monitor_start.connect(lambda: self.monitor(start=True))
                self.content[key].sig_monitor_stop.connect(lambda: self.monitor(start=False))
                self.content[key].sig_check_quota.connect(self.check_quota)
            else:
                pass

            if key == 'Notification_widget':
                self.content[key].update_telegram()
                self.content[key].update_email()
                self.content[key].update()
                self.content[key].sig_stop.connect(self.stop)
                timer = QTimer(self)
                timer.setInterval(20000)
                timer.timeout.connect(self.content[key].get_telegram_messages)
                timer.start()
            else:
                pass

            if key == 'Plot per micrograph' or \
                    key == 'Plot histogram' or \
                    key == 'Show images':
                self.content[key].worker.sig_data.connect(
                    self.content[key].update_figure
                    )
                self.content[key].worker.sig_reset.connect(
                    self.content[key].reset_plot
                    )
                self.content[key].worker.sig_visible.connect(
                    self.content[key].set_visibility
                    )
            else:
                pass
        self.content['Global'].set_global(global_dict)
        for key in self.content:
            try:
                self.content[key].sig_change_use_movie.connect(self.content['Global'].emit_global)
            except AttributeError:
                pass
        self.content['Copy'].emit_signals()

        return error_list

    @pyqtSlot(object, str)
    def hide_tab(self, sender, text):
        current_key = sender.parent().name
        try:
            for idx, widget in reversed(list(enumerate(self.content[current_key].widgets))):
                if text in ('Later', 'False'):
                    val = True
                else:
                    val = text == widget.name
                index = self.content[current_key].indexOf(widget)
                self.content[current_key].setTabEnabled(index, val)
                if val:
                    self.content[current_key].setCurrentIndex(index)
            self.content[current_key].order_tabs()
        except KeyError:
            pass

    def check_quota(self):
        """
        Check the quota for the project and scratch directory.

        Arguments:
        None

        Return:
        None
        """
        general_settings = self.content['Output'].get_settings()
        notification_settings = self.content['Notification'].get_settings()
        global_settings = {
            'Output': general_settings[0],
            'Notification': notification_settings[0]
            }
        self.workers['mount'].sig_set_settings.emit(global_settings)

    def load(self, file_name=None):
        """
        Load settings from settings file.

        Arguments:
        file_name - Name of the file (default None)

        Return:
        None
        """
        if file_name is None:
            file_name = QFileDialog.getOpenFileName(
                caption='Load settings',
                directory=self.path,
                options=QFileDialog.DontUseNativeDialog
                )

            file_name = file_name[0]

            if not file_name:
                return
            else:
                pass

        if file_name.endswith('.txt'):
            pass
        elif os.path.exists(file_name):
            pass
        else:
            file_name = '{0}.txt'.format(file_name)


        settings = []
        with open(file_name, 'r') as read:
            for line in read:
                line = line.replace('\n', '')
                key, *value = line.split('\t')
                if isinstance(value, list):
                    settings.append([key, *value])
                else:
                    settings.append([key, value])

        settings = self.settings_to_dict(settings=settings)
        self.set_settings(settings=settings)

    def set_design(self, settings):
        """
        Load settings from settings file.

        Arguments:
        settings - Settings as dictionary.

        Return:
        None
        """
        for key in settings:
            try:
                self.content[key].set_design(settings[key])
            except KeyError:
                try: # This block has been introduced for backwards compatibility changes.
                    new_key = key.replace(' v', ' >=v')
                    self.content[new_key].set_design(settings[key])
                except KeyError:
                    print('Key', key, 'no longer exists')
                    continue
            except AttributeError:
                pass

    def set_settings(self, settings):
        """
        Load settings from settings file.

        Arguments:
        settings - Settings as dictionary.

        Return:
        None
        """
        keys = list(settings.keys())

        # Move Global to the very end if available to let the dependencies work properly
        try:
            keys.append(keys.pop(keys.index('Global')))
        except ValueError:
            pass
        for key in keys:
            if key in ('End',):
                continue
            else:
                try:
                    self.content[key].set_settings(settings[key])
                except KeyError:
                    try: # This block has been introduced for backwards compatibility changes.
                        new_key = key.replace(' v', ' >=v')
                        self.content[new_key].set_settings(settings[key])
                    except KeyError:
                        print('Key', key, 'no longer exists')
                        continue

    @staticmethod
    def settings_to_dict(settings):
        """
        Make the settings readable for the widgets set settings method.

        Arguments:
        settings - Settings as dictionary.

        Return:
        None
        """
        settings_dict = {}
        idx = -1
        key = None
        while idx < len(settings)-1:
            idx += 1
            if settings[idx][0] == '###':
                key = settings[idx][1]
                if key == 'Frames':
                    settings_dict[key] = []
                else:
                    settings_dict[key] = {}
                continue

            if len(settings[idx]) == 1:
                settings[idx].append('')

            if key is None:
                continue
            elif key == 'Frames':
                setting = {}
                setting[settings[idx][0]] = settings[idx][1]
                for i in range(3):
                    idx += 1
                    setting[settings[idx][0]] = settings[idx][1]
                settings_dict[key].append(setting)
            else:
                if key == 'Notification_widget':
                    settings_dict[key].update({settings[idx][0]: [
                        settings[idx][1],
                        settings[idx][2]
                        ]})
                else:
                    settings_dict[key].update({
                        settings[idx][0]: settings[idx][1]
                        })

        return settings_dict

    def save(self, file_name=None, temp=False, interactive=False, do_message=True):
        """
        Save GUI status to file.

        Arguments:
        file_name - File name to save settings to.
        temp - File is a temporary save file.

        Return:
        True, if saving was succesful.
        """
        if file_name is None:
            interactive = True
            file_name = QFileDialog.getSaveFileName(
                caption='Save settings',
                directory=self.path,
                options=QFileDialog.DontUseNativeDialog,
                filter="Text files (*.txt)"
                )

            file_name = file_name[0]

            if not file_name:
                return None
            else:
                pass
        else:
            pass

        if file_name.endswith('.txt'):
            pass
        else:
            file_name = '{0}.txt'.format(file_name)

        # Do not override settings
        if interactive:
            pass
        else:
            old_filename, old_ext = os.path.splitext(file_name)
            for number in range(9999):
                file_name = '{0}_{1:05d}{2}'.format(old_filename, number, old_ext)
                if os.path.exists(file_name):
                    continue
                else:
                    break

        error = False
        try:
            with open(file_name, 'w') as write:
                for key in self.content:
                    if key == 'Mount':
                        continue
                    else:
                        pass
                    try:
                        settings = self.content[key].get_settings()
                    except AttributeError:
                        continue
                    if settings is not None:
                        write.write('###\t{0}\n'.format(key))
                        for entry in settings:
                            for key_entry in entry:
                                if key_entry == 'SSH password':
                                    continue
                                write.write(
                                    '{0}\t{1}\n'.format(key_entry, entry[key_entry])
                                    )
                    else:
                        error = True
                        message = 'Setting of {0} not valid!'.format(key)
                        tu.message(message)
                        continue
                write.write('###\tEnd\n')
                write.write('The\tEnd\n')
        except Exception as e:
            error = False
            tu.message('Failed to save settings: {0}'.format(e))
        else:
            message_pass = 'Valid settings saved to {0}'.format(file_name)
            message_error = 'Invalid setting detected! Saveing failed!'
            if do_message:
                if error:
                    os.remove(file_name)
                    tu.message(message_error)
                    print(message_error)
                    return False
                else:
                    print(message_pass)
                    if temp:
                        pass
                    else:
                        tu.message(message_pass)
                    return file_name
            else:
                if error:
                    return False, message_error
                else:
                    return file_name, message_pass

    def monitor(self, start):
        """
        Start the TranSPHIRE monitor processing.

        Arguments:
        start - True if start, False if stop

        Returns:
        None
        """
        self.enable(False)
        self.content['Button'].start_monitor_button.setEnabled(False)
        self.content['Button'].stop_monitor_button.setEnabled(False)

        if start:
            self.workers['plotting'].sig_reset_list.emit()
            settings, _, _ = self.get_start_settings(monitor=True)
            if settings is None:
                tu.message('Please fill non emtpy entries.')
                self.enable(True)
                self.content['Button'].start_monitor_button.setEnabled(True)
                start = False
            elif not os.path.exists(settings['project_folder']):
                tu.message('Project folder does not exists. Cannot monitor session.')
                self.enable(True)
                self.content['Button'].start_monitor_button.setEnabled(True)
                start = False
            else:
                self.workers['process'].sig_start.emit(settings, {})
                self.content['Button'].start_button.setEnabled(False)
                self.content['Button'].stop_button.setEnabled(False)
        else:
            self.workers['process'].stop = True

        if start:
            self.content['Button'].start_monitor_button.setVisible(False)
            self.content['Button'].stop_monitor_button.setVisible(True)
            self.content['Button'].stop_monitor_button.setEnabled(True)
            self.content['Retrain'].sig_start.emit(copy.deepcopy(settings))

    def _extract_settings(self, key, settings, error_list, check_list, external_files):
        try:
            settings_widget = self.content[key].get_settings()
        except AttributeError:
            return
        else:
            settings[key] = {}

        try:
            input_file_names = self.content[key].get_input_names()
        except AttributeError:
            input_file_names = []

        if settings_widget is None:
            self.enable(True)
            error_list.append('{0} needs to have problems fixed!'.format(key))
            return None
        elif key == 'Frames':
            settings_motion = {}
        else:
            pass

        if key == 'Frames':
            skip_name_list = []
        else:
            skip_name_list = tu.get_function_dict()[key]['allow_empty']

        non_global_names_with_global = {}
        for entry in settings_widget:
            for name in sorted(list(entry.keys())):
                if name.endswith('_global'):
                    if entry[name][0] is not None and entry[name][1] and key != 'Global':
                        non_global_names_with_global[entry[name][0]] = name.split('_global')[0]
                    del entry[name]
                    continue

                if key in check_list:
                    if name == 'Number of feedbacks' and int(entry[name]) > self.n_feedbacks:
                        error_list.append(
                            '{0}:{1} is not allowed to be larger than the specified number: {2}!\nCheck the start settings of TranSPHIRE in case you want more feedbacks!'.format(
                                key,
                                name,
                                self.n_feedbacks
                                )
                            )
                    if not entry[name] and name not in skip_name_list:
                        error_list.append(
                            '{0}:{1} is not allowed to be empty!'.format(
                                key,
                                name
                                )
                            )
                    else:
                        pass
            else:
                pass

        for global_key, local_key in non_global_names_with_global.items():
            if global_key == 'GPU':
                for entry in settings_widget:
                    try:
                        gpu_splits = int(entry['GPU SPLIT'][0])
                    except KeyError:
                        gpu_splits = 0

                    if gpu_splits != 0:
                        new_gpu = []
                        for i in range(gpu_splits):
                            for gpu_idx in entry[local_key].split():
                                new_gpu.append('{}_{}'.format(gpu_idx, i))
                        entry[local_key] = ' '.join(new_gpu)
                    try:
                        del entry['GPU SPLIT']
                    except KeyError:
                        pass

        for local_key in input_file_names:
            external_files.setdefault(key, {})[local_key] = {}

        for idx, entry in enumerate(settings_widget):
            if key == 'Frames':
                settings_motion[idx] = entry
            else:
                settings[key].update(entry)

        if key == 'Frames':
            settings['motion_frames'] = settings_motion
        else:
            pass

    @pyqtSlot()
    def get_start_settings(self, monitor=False):
        """
        Start TranSPHIRE processing.

        Arguments:
        None

        Return:
        None
        """
        self.enable(False)
        settings = {}
        settings['Monitor'] = monitor
        # Load settings to pass them to the working threads
        error_list = []
        check_list = [
            'Output',
            'Notification'
            ]
        first_round = ['Global']
        for setting in self.content['Copy'].get_settings():
            for value in setting.values():
                if isinstance(value, str) and value not in ('False', 'Later', 'True'):
                    check_list.append(value)

        external_files = {}
        for key in first_round:
            self._extract_settings('Global', settings, error_list, check_list, external_files)
        for key in self.content:
            if key in first_round:
                continue
            self._extract_settings(key, settings, error_list, check_list, external_files)

        settings['motion_frames'] = {
            '0': {
                'first': 1,
                'last': -1,
                'dw': True,
                'default': True
                }
            }
        if error_list and not monitor:
            tu.message('\n'.join(error_list))
            self.enable(True)
            return None, None, None
        else:
            pass

        # Get mount information
        for key in settings['Mount']:
            device_name = key.replace(' ', '_')
            save_file = os.path.join(self.settings_folder, device_name)
            try:
                with open(save_file, 'r') as read:
                    lines = read.readlines()
            except FileNotFoundError:
                continue
            for line in lines:
                name = line.split('\t')[0]
                settings['user_{0}'.format(device_name)] = name
        settings['user_Later'] = None

        if not re.match(
                self.project_name_pattern,
                settings['Output']['Project name']
                ) and not monitor:
            self.enable(True)
            tu.message(
                'Project name needs to match pattern:\n{0}\n For example: {1}'.format(
                    self.project_name_pattern,
                    self.project_name_pattern_example
                    )
                )
            return None, None, None
        else:
            pass

        # Project folder names
        settings['project_base'] = os.path.join(
            settings['Output']['Project directory'],
            settings['Output']['Project name']
            )
        if not os.path.exists(settings['project_base']) and monitor:
            self.enable(True)
            tu.message('Project needs to exists in order to start Monitor mode')
            return None, None, None

        settings['project_folder'] = os.path.join(
            settings['project_base'],
            'TranSPHIRE_results'
            )

        settings['scratch_folder'] = os.path.join(
            settings['Output']['Scratch directory'],
            settings['Output']['Project name'],
            'TranSPHIRE_results'
            )
        folder_dict = {
            'log_folder': 'XXX_Log_files',
            'queue_folder': 'XXX_Queue_files',
            'error_folder': 'XXX_Error_files',
            'tar_folder': 'XXX_Tar_file_folder',
            'stack_folder': '000_Import',
            'meta_folder': '000_Import_meta',
            'set_folder_raw': 'XXX_Settings',
            'restart_backup_folder': 'XXX_Restart_Backup',
            'software_meta_folder': '000_Session_meta',
            }
        for key, value in folder_dict.items():
            settings[key] = os.path.join(
                settings['project_folder'],
                value
                )

        continue_mode = os.path.exists(settings['restart_backup_folder'])

        settings['do_feedback_loop'] = int(settings['Output']['Number of feedbacks'])
        settings['feedback_file'] = os.path.join(settings['log_folder'], 'feedback_log')
        settings['spot_file'] = os.path.join(settings['log_folder'], 'spot_dict.txt')
        settings['data_frame'] = os.path.join(settings['project_folder'], 'data_frame.csv')
        settings['external_log'] = os.path.join(settings['set_folder_raw'], 'external_files.json')
        settings['translation_file'] = os.path.join(settings['project_folder'], 'Valid_micrographs_info.txt')
        settings['translation_file_bad'] = os.path.join(settings['project_folder'], 'Discarded_micrographs_info.txt')

        settings['current_set'] = datetime.datetime.now().strftime('%Y_%m_%d_%H_%M_%S')

        settings['restart_backup_folder'] = os.path.join(settings['restart_backup_folder'], settings['current_set'])
        settings['set_folder'] = os.path.join(settings['set_folder_raw'], settings['current_set'])
        folder_dict['set_folder'] = settings['set_folder']
        folder_dict['restart_backup_folder'] = settings['restart_backup_folder']

        # Move Error files from previous run
        if continue_mode:
            tu.mkdir_p(settings['restart_backup_folder'])
            try:
                tu.copy(
                    settings['error_folder'],
                    settings['restart_backup_folder'],
                    )
                shutil.rmtree(settings['error_folder'])
            except FileNotFoundError:
                pass
            # Copy Queue files from previous run
            try:
                tu.copy(
                    settings['queue_folder'],
                    settings['restart_backup_folder'],
                    )
            except FileNotFoundError:
                pass

        names = [
            entry.replace('_entries', '')
            for entry in settings['Copy']
            if entry.endswith('_entries') and
            entry.replace('_entries', '').replace('_', ' ') in settings['Copy']
            ]
        idx = 0
        settings['plot_emit'] = []
        for entry in names:
            base_dir2 = None
            no_feedback = False
            if 'copy_to_' in entry.lower():
                base_dir = self.mount_directory
                no_feedback = True
            else:
                idx += 1
                base_dir = settings['project_folder']
                base_dir2 = settings['scratch_folder']

            entry_name = '{}_entries'.format(entry)
            prog_name_entries = settings['Copy'][entry_name] if entry_name in settings['Copy'] else [entry]
            for prog_name in prog_name_entries:
                if 'copy_to_' in entry.lower():
                    folder_name = prog_name.replace(' ', '_').replace('>=', '')
                else:
                    folder_name = '{0:03d}_{1}'.format(idx, prog_name.replace(' ', '_').replace('>=', ''))

                for index in range(int(settings['Output']['Number of feedbacks']) + 1):
                    if index == 0:
                        folder_name_tmp = folder_name
                    elif no_feedback:
                        continue
                    else:
                        folder_name_tmp = os.path.join(
                            '000_Feedback_results',
                            '{0}_feedback_{1}'.format(
                                folder_name,
                                int(settings['Output']['Number of feedbacks']) - index + 1
                                )
                            )

                    folder_setting_name = '{0}_folder_feedback_{1}'.format(entry.lower(), index)
                    folder_setting = os.path.join(base_dir, folder_name_tmp)
                    prog_name_feedback = '{} feedback {}'.format(prog_name, int(settings['Output']['Number of feedbacks']) - index + 1) if index != 0 else prog_name
                    settings['plot_emit'].append([prog_name, prog_name_feedback, entry, folder_setting])

                    if prog_name == settings['Copy'][entry.replace('_', ' ')]:
                        settings[folder_setting_name] = folder_setting
                        if base_dir2 is not None:
                            settings['scratch_{0}_folder_feedback_{1}'.format(entry.lower(), index)] = os.path.join(
                                base_dir2,
                                folder_name_tmp
                                )

        return settings, folder_dict, external_files

    @pyqtSlot()
    def start(self):
        settings, folder_dict, external_files = self.get_start_settings()

        # Check for continue mode
        if settings is None:
            return None
        elif os.path.exists(settings['project_folder']):
            result, restart_dict = self.continue_dialog(
                text1='Output project folder already exists!',
                text2='Do you really want to continue the old run?\nType: "YES!"',
                is_restart=True,
                settings=settings,
                )
        else:
            result = True
            restart_dict = {}

        # Start or stop procedure
        if result:
            # Create project and settings folder
            folder_names = ['project_folder', 'scratch_folder']
            folder_names.extend(list(folder_dict.keys()))
            for name in folder_names:
                try:
                    tu.mkdir_p(settings[name])
                except PermissionError as e:
                    print(e)
                    tu.message('You do not have permission to write to your specified project directory! Check the Terminal for the actual error message!')
                    self.enable(True)
                    return None
                except FileNotFoundError as e:
                    print(e)
                    tu.message('Project name cannot be empty! Check the Terminal for the actual error message!')
                    self.enable(True)
                    return None

            for key, local_dict in external_files.items():
                if key in ('Path', 'Compress cmd'):
                    continue
                for local_key, value in local_dict.items():
                    if settings[key][local_key]:
                        value['old_file'] = settings[key][local_key]
                        value['new_file'] = os.path.join(
                            settings['set_folder'],
                            os.path.basename(value['old_file'])
                            )
                        try:
                            tu.copy(value['old_file'], value['new_file'])
                        except FileNotFoundError:
                            tu.message("Input file {} not available!".format(value['old_file']))
                            self.enable(True)
                            return None
                        settings[key][local_key] = 'external_log|||{}'.format(local_key)

            try:
                with open(settings['external_log'], 'r') as read:
                    current_data = json.load(read)
            except FileNotFoundError:
                current_data = {}
            current_data[settings['current_set']] = external_files
            with open(settings['external_log'], 'w') as write:
                json.dump(current_data, write, indent=1)

            self.content['Button'].start_button.setVisible(False)
            self.content['Button'].start_button.setEnabled(False)
            self.content['Button'].stop_button.setVisible(True)
            self.content['Button'].stop_button.setEnabled(True)
            self.content['Button'].start_monitor_button.setEnabled(False)
            self.content['Button'].stop_monitor_button.setEnabled(False)
            self.workers['plotting'].sig_reset_list.emit()
            _, message = self.save(
                file_name=os.path.join(
                    settings['set_folder'],
                    settings['Output']['Project name']
                    ),
                do_message=False
                )
            _, message = self.save(
                file_name=os.path.join(
                    self.project_settings_folder,
                    settings['Output']['Project name']
                    ),
                do_message=False
                )
            with open(os.path.join(settings['set_folder'], 'used_settings.json'), 'w') as write:
                json.dump(settings, write, indent=1)
            settings_copy = copy.deepcopy(settings)
            self.workers['process'].sig_start.emit(settings, restart_dict)
            self.workers['mount'].set_settings(settings=settings_copy)
            self.save_temp_settings()
            tu.message(message)
            self.content['Retrain'].sig_start.emit(settings_copy)
        else:
            tu.message('Input needs to be "YES!" to work')
            self.enable(True)

    def continue_dialog(self, text1, text2, is_stop=False, is_restart=False, settings=None):
        """
        Check if the user wants to run the continue mode.

        Arguments:
        text1 - Dialog window name.
        text2 - Text of the dialog.

        Return:
        True, if the input is YES!
        """
        if is_restart:
            restart_names = tu.get_unique_types()
        else:
            restart_names = np.array([])
        dialog = InputBox(
            is_password=False,
            parent=self,
            is_stop=is_stop,
            restart_names=restart_names,
            settings=settings,
            )
        dialog.setText(text1, text2)
        result = dialog.exec_()

        text = dialog.getText()
        if result and is_restart:
            return bool(text == 'YES!'), dialog.get_restart_dict()
        elif result and is_stop:
            return bool(text == 'YES!'), dialog.abort.isChecked()
        elif result:
            return bool(text == 'YES!')
        elif is_restart:
            return False, {}
        elif is_stop:
            return False, False
        else:
            return False

    @pyqtSlot()
    def stop_dialog(self):
        """
        Check if the user really wants to stop the process.

        Arguments:
        None

        Return:
        None
        """
        result, abort = self.continue_dialog(
            text1='Do you really want to stop?',
            text2='Do you really want to stop!\nType: "YES!"',
            is_stop=True,
            )
        if result:
            self.stop(abort=abort)
        else:
            tu.message('Input needs to be "YES!" to work')

    @pyqtSlot()
    def stop(self, abort=False):
        """
        Stop the process.

        Arguments:
        None

        Return:
        None
        """
        if abort:
            self.workers['process'].abort = True
        if self.workers['process'].stop == False:
            self.workers['process'].stop = True
            self.content['Button'].start_monitor_button.setEnabled(False)
            self.content['Button'].stop_monitor_button.setEnabled(False)
            self.content['Button'].start_button.setEnabled(False)

    @pyqtSlot()
    def _finished(self):
        """
        Rename the Stop Button to start and enable everything.

        Arguments:
        None

        Return:
        None
        """
        self.content['Button'].stop_button.setVisible(False)
        self.content['Button'].start_button.setVisible(True)
        self.content['Button'].start_button.setEnabled(True)
        self.content['Button'].stop_button.setEnabled(False)

        self.content['Button'].stop_monitor_button.setVisible(False)
        self.content['Button'].start_monitor_button.setVisible(True)
        self.content['Button'].start_monitor_button.setEnabled(True)
        self.content['Button'].stop_monitor_button.setEnabled(False)
        self.enable(True)

    @pyqtSlot(bool)
    def enable(self, var, use_all=False):
        """Enable or disable widgets

        Arguments:
        var - Enable status of the widgets.
        use_all - Disable/Enable everything (Default False)

        Return:
        None
        """
        for key in self.content:
            try:
                self.content[key].enable(var=var, use_all=use_all)
            except AttributeError:
                continue

    def closeEvent(self, event):
        """
        Quit threads before close and check if the process is still running

        Arguments:
        event - QCloseEvent.

        Return:
        None
        """
        if self.content['Button'].stop_button.isVisible() or \
                self.content['Button'].stop_monitor_button.isVisible():
            event.ignore()
            tu.message('First stop the program before closing')
            return None
        elif not self.save_temp_settings():
            result = tu.question(
                head='Error saving file!',
                text='Wrong setting detected! Quit without saving?',
                )
            # Result is true if the answer is Yes
            if result:
                pass
            else:
                event.ignore()
                return None
        else:
            pass

        for entry in self.types:
            if self.threads[entry] is not None:
                self.threads[entry].stop = True

                if entry not in self.default_types:
                    self.timers[entry].stop()

        print('Wait for threads to exit.')
        for thread_instance in self.threads.values():
            if thread_instance is not None:
                thread_instance.quit()
                thread_instance.wait()

        print('Wait for thread mount')
        for key in self.content['Mount'].content:
            thread = self.mount_thread_list[key]['thread']
            calculator = self.mount_thread_list[key]['object']
            calculator.kill_thread = True
            thread.quit()
            thread.wait()
        print('Bye Bye')
        super(MainWindow, self).closeEvent(event)

# Objects
from .mountworker import MountWorker
from .processworker import ProcessWorker
from .inputbox import InputBox
from .plotworker import PlotWorker
from .separator import Separator
from .tabdocker import TabDocker
from .mountcalculator import MountCalculator



from PyQt5.QtWidgets import QPlainTextEdit, QWidget, QHBoxLayout, QVBoxLayout, QPushButton, QLineEdit
from PyQt5.QtGui import QTextOption, QTextCursor
from PyQt5.QtCore import pyqtSlot, QTimer
import glob
import os
import getpass

from . import transphire_utils as tu
from . import logviewerdialog

class LogViewer(QWidget):

    def __init__(self, show_indicators=False, indicator='', file_name='', parent=None):
        super(LogViewer, self).__init__(parent)

        # Setup layout
        global_layout = QHBoxLayout(self)
        global_layout.setContentsMargins(0, 0, 0, 0)

        widget = QWidget(self)
        widget.setObjectName('logview')
        global_layout.addWidget(widget)

        layout = QVBoxLayout(widget)

        self.project_path = ''
        self.log_path = ''
        self.error_path = ''
        self.indicator_names = ('log', 'error', 'sys_log', 'notes')
        self.indicator = indicator

        self.text = QPlainTextEdit(widget)
        if file_name:
            self.text.setObjectName('dialog')
        else:
            self.text.setObjectName('status')
        self.text.setPlaceholderText('Welcome to TranSPHIRE!')
        self.text.setToolTip('Double click after starting TranSPHIRE in order to show more information')
        self.text.setReadOnly(True)
        self.text.setWordWrapMode(QTextOption.WrapAnywhere)
        layout.addWidget(self.text, stretch=1)

        self.file_name = file_name

        self.buttons = {}
        if show_indicators:

            layout_h1 = QHBoxLayout()

            for entry in self.indicator_names:
                template = '{0}: {{0}}'.format(entry)
                self.buttons[entry] = [QPushButton(self), template]
                self.buttons[entry][0].setObjectName('button_entry')
                self.buttons[entry][0].clicked.connect(self.my_click_event)

                layout_h1.addWidget(self.buttons[entry][0])
                self.increment_indicator(entry, '0')

            layout_h1.addStretch(1)
            layout.addLayout(layout_h1)

            self.change_state(False)

        self.update_plain_text(force=True)
        self.timer = QTimer(self)
        self.timer.setInterval(1000)
        self.timer.timeout.connect(self.update_plain_text)
        self.timer.start()

        if self.indicator == 'notes':
            layout_h = QHBoxLayout()
            layout_h.setContentsMargins(0, 0, 0, 0)
            self.input_edit = QLineEdit('', self)
            submit_button = QPushButton('Submit', self)
            submit_button.clicked.connect(self.submit_text)
            layout_h.addWidget(self.input_edit, stretch=1)
            layout_h.addWidget(submit_button)
            layout.addLayout(layout_h)

    @pyqtSlot()
    def update_plain_text(self, force=False):
        if self.file_name and os.path.exists(self.file_name):
            with open(self.file_name, 'r') as read:
                text = read.read()
            if force:
                self.reset_plain_text(text)
            elif text.replace('\n', '').replace(' ', '') != self.text.toPlainText().replace('\n', '').replace(' ', ''):
                self.reset_plain_text(text)

    def reset_plain_text(self, text):
        self.text.setPlainText(text)
        cursor = self.text.textCursor()
        cursor.movePosition(QTextCursor.End)
        self.text.setTextCursor(cursor)

    @pyqtSlot()
    def submit_text(self):
        self.appendPlainText(self.input_edit.text(), indicator='notes', user=True)
        self.input_edit.setText('')

    def appendPlainText(self, text, indicator='log', user=False):
        text_raw = tu.create_log(text)
        prefix, suffix = text_raw.split(' => ', 1)
        if user:
            text = '{}\n{}: {}\n'.format(prefix, getpass.getuser(), suffix)
        else:
            text = '{}\n{}\n'.format(prefix, suffix)
        try:
            with open(self.file_name, 'a+') as write:
                write.write(text)
        except IOError:
            pass
        self.text.appendPlainText(text)
        cursor = self.text.textCursor()
        cursor.movePosition(QTextCursor.End)
        self.text.setTextCursor(cursor)
        print(text)
        if self.project_path:
            self.increment_indicator(indicator)

    def increment_indicator(self, indicator, text=''):
        if indicator in self.indicator_names:
            button, template = self.buttons[indicator]
            if text:
                cur_text = text
            else:
                cur_text = str(1 + int(self.get_indicator(indicator)))

            button.setText(template.format(cur_text))
            button.setToolTip(template.format(text))
            if self.get_indicator(indicator) == '0':
                button.setStyleSheet('')
            else:
                button.setStyleSheet(tu.get_style('changed'))
        else:
            assert False, indicator

    def get_indicator(self, indicator):
        if indicator in self.indicator_names:
            return self.buttons[indicator][0].text().split(':')[-1].strip()
        else:
            assert False, indicator

    @pyqtSlot()
    def my_click_event(self, event=None):
        if not self.project_path:
            return None

        sender = self.sender()
        sender_text = sender.text().split(':')[0].strip()
        is_notes = False
        file_path = self.log_path

        if sender_text == 'log':
            file_names = ['log.txt']

        elif sender_text == 'notes':
            is_notes = True
            file_names = ['notes.txt']

        elif sender_text == 'sys_log':
            file_names = ['sys_log.txt']

        elif sender_text == 'error':
            file_names = [os.path.basename(entry) for entry in glob.glob(os.path.join(self.error_path, '*'))]
            file_path = self.error_path

        else:
            assert False, sender.text()

        if not is_notes:
            self.increment_indicator(sender_text, '0')

        sender.setEnabled(False)
        QTimer.singleShot(5000, lambda: sender.setEnabled(True))

        dialog = logviewerdialog.LogViewerDialog(self)
        for file_name in file_names:
            dialog.add_tab(
                LogViewer(file_name=os.path.join(file_path, file_name), indicator=sender_text, parent=self),
                os.path.basename(file_name),
                )
        dialog.show()

    @pyqtSlot(str, str, str)
    def set_project_path(self, project_path, log_path, error_path):
        self.project_path = project_path
        self.log_path = log_path
        self.error_path = error_path
        state = True
        if not self.project_path:
            state = False
            self.file_name = ''
        elif not self.file_name:
            self.file_name = os.path.join(self.log_path, 'log.txt')
            self.update_plain_text(force=True)
        self.change_state(state)

    def change_state(self, state):
        self.text.blockSignals(not state)
        for button, _ in self.buttons.values():
            button.setEnabled(state)
            button.blockSignals(not state)




import os
import glob
import shutil
from PyQt5.QtWidgets import QDialog, QVBoxLayout, QPushButton, QWidget, QComboBox, QLabel
from PyQt5.QtCore import pyqtSlot

from . import transphire_utils as tu

class TemplateDialog(QDialog):
    """
    TemplateDialog dialog.
    Dialog used to enter the template.

    Inherits from:
    QDialog
    """

    def __init__(self, settings_directory, add_remove=True, parent=None):
        super(TemplateDialog, self).__init__(parent)

        self.settings_directory = settings_directory
        self.template = None
        central_raw_layout = QVBoxLayout(self)
        central_raw_layout.setContentsMargins(0, 0, 0, 0)
        central_widget_raw = QWidget(self)
        central_widget_raw.setObjectName('central_raw')
        central_raw_layout.addWidget(central_widget_raw)

        central_layout = QVBoxLayout(central_widget_raw)
        central_widget = QWidget(self)
        central_widget.setObjectName('central')
        central_layout.addWidget(central_widget)

        layout = QVBoxLayout(central_widget)

        self.templates = sorted([os.path.basename(entry) for entry in glob.glob(os.path.join(settings_directory, '*')) if os.path.isdir(entry)])

        layout.addWidget(QLabel('Available templates:'))

        self.combo_box = QComboBox(parent=self)
        self.combo_box.clear()
        self.combo_box.addItems([entry for entry in self.templates if entry != 'SHARED'])
        layout.addWidget(self.combo_box)

        if add_remove:
            button = QPushButton('New template', self)
            button.clicked.connect(self.add_template)
            layout.addWidget(button)

            button = QPushButton('Copy current template', self)
            button.clicked.connect(self.copy_template)
            layout.addWidget(button)

            button = QPushButton('Remove current template', self)
            button.clicked.connect(self.remove_template)
            layout.addWidget(button)

        button = QPushButton('Choose current template', self)
        button.clicked.connect(self.choose_template)
        layout.addWidget(button)

    @pyqtSlot()
    def choose_template(self):
        self.template = self.combo_box.currentText()
        self.accept()

    @pyqtSlot()
    def remove_template(self):
        text = self.combo_box.currentText()
        if text == 'DEFAULT':
            tu.message('Default template cannot be deleted!')
            return None
        dialog = InputBox(is_password=False, parent=self)
        dialog.setText('Confirm deletion', 'Do you really want to remove delete template {0}? Type in "YES!"'.format(text))
        result = dialog.exec_()
        if result:
            response = dialog.getText()
            if response == 'YES!':
                try:
                    shutil.rmtree(os.path.join(self.settings_directory, text))
                except FileNotFoundError:
                    tu.message('Something is wrong! Template directory could not be removed!')
                else:
                    self.templates.remove(text)
                    self.combo_box.clear()
                    self.combo_box.addItems([entry for entry in self.templates if entry != 'SHARED'])
            else:
                tu.message('Input needs to be "YES!" to work')

    @pyqtSlot()
    def add_template(self):
        dialog = InputBox(is_password=False, parent=self)
        dialog.setText('New template', 'Template name:')
        result = dialog.exec_()

        if result:
            text = dialog.getText().strip()
            if ' ' in text:
                tu.message('There are not whitespaces allowed in the template name!')
            elif 'shared' == text.lower():
                tu.message('Shared is a protected namespace. Please choose another name.')
            elif text in self.templates:
                tu.message('Template name already exists! Please choose another one!')
            else:
                try:
                    os.mkdir(os.path.join(self.settings_directory, text))
                except IOError:
                    tu.message('Something is wrong! Template directory already exists!')
                else:
                    self.templates.append(text)
                    self.combo_box.clear()
                    self.combo_box.addItems([entry for entry in self.templates if entry != 'SHARED'])
                    self.combo_box.setCurrentText(text)

    @pyqtSlot()
    def copy_template(self):
        current_template = self.combo_box.currentText()
        if not current_template:
            tu.message('No template selected!')
            return None

        dialog = InputBox(is_password=False, parent=self)
        dialog.setText('New template', 'Template name:')
        result = dialog.exec_()

        if result:
            text = dialog.getText()
            if ' ' in text:
                tu.message('There are not whitespaces allowed in the template name!')
            elif text in self.templates:
                tu.message('Template name already exists! Please choose another one!')
            else:
                try:
                    shutil.copytree(
                        os.path.join(self.settings_directory, current_template),
                        os.path.join(self.settings_directory, text)
                        )
                except IOError as e:
                    print(e)
                    tu.message('Something is wrong! Template directory already exists!')
                else:
                    self.templates.append(text)
                    self.combo_box.clear()
                    self.combo_box.addItems(self.templates)
                    self.combo_box.setCurrentText(text)

from .inputbox import InputBox




"""
    TranSPHIRE is supposed to help with the cryo-EM data collection
    Copyright (C) 2017 Markus Stabrin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import shutil
import sys
import os
import pexpect as pe
from PyQt5.QtCore import pyqtSignal, QObject, pyqtSlot, QThread


class MountWorker(QObject):
    """
    Mounting and unmounting shared devices.

    Inherits from:
    QObject

    Buttons:
    None

    Signals:
    sig_mount_hdd - Signal connected to mount HDD (device|str)
    sig_mount - Signal connected to mount a mount point (device|str, user|str, password|str, folder|str, server|str, typ|str, domain|str, version|str, sec|str, gid|str)
    sig_umount - Signal connected to unmount a mount point (device_folder|str, device|str, thread|object)

    sig_success - Signal emitted, if a task was a success (text|str, device|str, color|str)
    sig_error - Signal emitted, if a task was a failure (text|str, device|str)
    sig_info - Signal emitted, to show text in a text box (text|str)
    sig_notification - Signal emitted, to send a notification message (text|str)

    sig_add_save - Signal connected to add a save file to the dictionary (device|str, ss_address|str, quota_command|str, is_right_quota|str, quota|str)
    sig_load_save - Signal connected to load data from save file (No object)
    sig_refresh - Signal connected to recalculate quota (No object)
    sig_quota - Signal emitted, to refresh the quota status in the GUI (text|str, device|str, color|str)
    sig_set_settings - Signal connected set quota related settings (settings|object)

    sig_calculate_ssh_quota - Signal emitted to calculate the quota via ssh (user|str, folder|str, device|str, mount_folder|str, ssh_dict|object, quota_command_dict|object, password_dict|object)
    sig_calculate_df_quota - Signal emitted to calculate the quota via system information (device|str, mount_folder|str)
    sig_calculate_get_quota - Signal emitted to calculate the quota brute force (device|str, total_quota|str, mount_folder|str)
    """
    sig_mount_hdd = pyqtSignal(str)
    sig_mount = pyqtSignal(str, str, str, str, str, str, str, str, str, str, str, str)

    sig_umount = pyqtSignal(str, str, str, object)

    sig_success = pyqtSignal(str, str, str)
    sig_error = pyqtSignal(str, str)
    sig_info = pyqtSignal(str)
    sig_notification = pyqtSignal(str)

    sig_add_save = pyqtSignal(str, str, str, str, str)
    sig_load_save = pyqtSignal()
    sig_refresh = pyqtSignal()
    sig_quota = pyqtSignal(str, str, str)
    sig_set_settings = pyqtSignal(object)

    sig_calculate_ssh_quota = pyqtSignal(str, str, str, str, object, object, object)
    sig_calculate_df_quota = pyqtSignal(str, str)
    sig_calculate_get_quota = pyqtSignal(str, str, str)

    sig_set_folder = pyqtSignal(str, str)

    def __init__(self, password, settings_folder, mount_directory, parent=None):
        """
        Initialize object variables.

        Arguments:
        password - Sudo password
        settings_folder - Folder to save settings to
        mount_directory - Folder for mount points
        parent - Parent widget (default None)

        Return:
        None
        """
        super(MountWorker, self).__init__(parent)

        # Variables
        self.mount_directory = mount_directory
        self.password = password
        self.settings_folder = settings_folder
        self.save_files = {}
        self.password_dict = {}
        self.ssh_dict = {}
        self.quota_command_dict = {}
        self.is_right_quota_dict = {}
        self.quota_dict = {}
        self.refresh_count = {}
        self.refresh_billy = 0
        self.refresh_clr = 0
        self.project_directory = None
        self.project_quota_limit = None
        self.project_quota_warning = True
        self.scratch_directory = None
        self.scratch_quota_limit = None
        self.scratch_quota_warning = True
        self.abort_finished = False

        # Events
        self.sig_mount.connect(self.mount)
        self.sig_mount_hdd.connect(self.mount_hdd)
        self.sig_umount.connect(self.umount)
        self.sig_add_save.connect(self.add_save)
        self.sig_load_save.connect(self.load_save)
        self.sig_refresh.connect(self.refresh_quota)
        self.sig_set_settings.connect(self.set_settings)

        self.refresh_quota()

    @pyqtSlot(object)
    def set_settings(self, settings):
        """
        Set settings used by the worker.

        Arguments:
        settings - TranSPHIRE settings

        Return:
        None
        """
        if settings['Output']['Project directory']:
            self.project_directory = settings['Output']['Project directory']
        else:
            self.project_directory = '.'

        if settings['Notification']['Project quota warning (%)']:
            self.project_quota_limit = min(float(settings['Notification']['Project quota warning (%)']), 100)
        else:
            self.project_quota_limit = 95

        if settings['Output']['Scratch directory']:
            self.scratch_directory = settings['Output']['Scratch directory']
        else:
            self.scratch_directory = '.'

        if settings['Notification']['Scratch quota warning (%)']:
            self.scratch_quota_limit = min(float(settings['Notification']['Scratch quota warning (%)']), 100)
        else:
            self.scratch_quota_limit = 95

        self.refresh_quota()

    @pyqtSlot(str, str, str, str, str)
    def add_save(self, device, ssh_address, quota_command, is_right_quota, quota):
        """
        Add a save file to the dictonaries.

        Arguments:
        device - Mounted device name
        ssh_address - ssh adress
        quota_command - Command to calculate quota via ssh
        is_right_quota - True, if df is showing the right quota
        quota - Provided maximum quota

        Return:
        None
        """
        file_name = os.path.join(self.settings_folder, device)
        self.save_files[device] = file_name
        self.ssh_dict[device] = ssh_address
        self.quota_command_dict[device] = quota_command
        self.is_right_quota_dict[device] = is_right_quota
        self.quota_dict[device] = quota
        self.refresh_count[device] = 0
        if not os.path.exists(file_name):
            file_name = open(file_name, 'w')
            file_name.close()
        else:
            pass

    @pyqtSlot()
    def load_save(self):
        """
        Load connection status from the files

        Arguments:
        None

        Return:
        None
        """
        for key in self.save_files:
            with open(self.save_files[key], 'r') as read:
                line = read.readline().rstrip()
            if '\t' not in line:
                continue
            else:
                entry = line.split('\t')
                self.sig_success.emit(entry[0], key, 'lightgreen')
        self.refresh_quota()

    @pyqtSlot()
    def refresh_quota(self):
        """
        Refresh quota information.

        Arguments:
        None

        Return:
        None
        """
        self.check_connection()
        for key in self.save_files:
            with open(self.save_files[key], 'r') as read:
                lines = [line.rstrip('\n') for line in read.readlines()]

            # Continue if the file is empty
            if not lines:
                self.refresh_count[key] = 0
                self.sig_quota.emit('-- / --', key, 'white')
                continue
            else:
                pass

            for line in lines:
                user, folder, mount_folder, device, ssh_address, right_quota, quota, folder_from_root = line.split('\t')
                assert key == device

                # Only refresh quota after some time
                if self.refresh_count[key] == 0:
                    self.sig_quota.emit('Calculating...', key, 'lightgreen')
                    self.refresh_count[key] += 1
                    if ssh_address:
                        self.sig_calculate_ssh_quota.emit(
                            user,
                            folder,
                            device,
                            mount_folder,
                            self.ssh_dict,
                            self.quota_command_dict,
                            self.password_dict
                            )
                    elif right_quota == 'True':
                        self.sig_calculate_df_quota.emit(
                            key, mount_folder
                            )
                    else:
                        self.sig_calculate_get_quota.emit(
                            key,
                            quota,
                            mount_folder
                            )
                elif self.refresh_count[key] > 500:
                    self.refresh_count[key] = 0
                else:
                    self.refresh_count[key] += 1
                self.sig_set_folder.emit(device, os.path.join(folder_from_root, folder))

        if self.scratch_directory is not None:
            self.scratch_quota_warning = self.fill_quota_project_and_scratch(
                name='scratch',
                directory=self.scratch_directory,
                warning=self.scratch_quota_warning,
                quota_limit=self.scratch_quota_limit
                )
        else:
            pass
        if self.project_directory is not None:
            self.project_quota_warning = self.fill_quota_project_and_scratch(
                name='project',
                directory=self.project_directory,
                warning=self.project_quota_warning,
                quota_limit=self.project_quota_limit
                )
        else:
            pass

        self.check_connection()

    def fill_quota_project_and_scratch(self, name, directory, warning, quota_limit):
        """
        Refresh quota information for the project and scratch directory.

        Arguments:
        name - Name (project or scratch)
        directory - Directory to check
        warning - current warning status
        quota_limit - Limit of the quota to show a warning

        Return:
        Current warning status
        """
        try:
            total_quota = shutil.disk_usage(directory).total / 1e12
            used_quota = shutil.disk_usage(directory).used / 1e12
            free_quota = shutil.disk_usage(directory).free / 1e12

            # Decide if there is a quota warning
            limit = total_quota * (100 - quota_limit) / 100
            if warning:
                if free_quota < limit:
                    warning = False
                    message = 'Less than {0:.2f}Tb ({1:.2f}Tb) free on {2}!'.format(
                        limit,
                        free_quota,
                        name
                        )
                    self.sig_notification.emit(message)
                    self.sig_error.emit(message, 'None')
                else:
                    pass
            elif used_quota < limit * 0.9:
                warning = True
            else:
                pass
        except FileNotFoundError:
            self.sig_quota.emit('Not avilable', name, '#ff5c33')
            self.sig_success.emit('Not connected', name, '#ff5c33')

        else:
            self.sig_quota.emit('{0:.1f}TB / {1:.1f}TB'.format(used_quota, total_quota), name, 'lightgreen')
            self.sig_success.emit('Connected', name, 'lightgreen')
        return warning

    @pyqtSlot()
    def check_connection(self):
        """
        Check if a mount connection crashed

        Arguments:
        None

        Return:
        None
        """
        for key in self.save_files:
            with open(self.save_files[key], 'r') as read:
                line = read.readline().rstrip()

            if not line:
                continue
            else:
                entry = line.split('\t')
                mount_folder = '{0}'.format(entry[2])
                if not os.path.ismount(os.path.relpath(mount_folder)):
                    try:
                        os.rmdir(mount_folder)
                    except OSError as err:
                        try:
                            os.listdir(mount_folder)
                        except PermissionError:
                            pass
                        except OSError as err_os:
                            if 'Required key not available:' in str(err_os):
                                self.sig_notification.emit('Lost connection: {0}'.format(key))
                                with open(self.save_files[key], 'w') as write:
                                    write.write('')
                                self.sig_error.emit('Lost connection: {0}'.format(key), key)
                                self.refresh_quota()
                            else:
                                print('{0} - Host seems to be down! It may recover soon! You might need to manually unmount with sudo umount -l {0}.'.format(mount_folder))
                        else:
                            print('OSError caught in mount_folder: {0}'.format(mount_folder))
                            if self.password:
                                print(str(err).replace(self.password, 'SUDOPASSWORD'))
                            else:
                                print(str(err))
                    else:
                        self.sig_notification.emit('Lost connection: {0}'.format(key))
                        with open(self.save_files[key], 'w') as write:
                            write.write('')
                        self.sig_error.emit('Lost connection: {0}'.format(key), key)
                        self.refresh_quota()
                else:
                    pass

    @pyqtSlot(str)
    def mount_hdd(self, device):
        """
        Mount external HDD

        Arguments:
        device - Device name

        Return:
        None
        """
        test_name = 'hdd_test'
        mount_folder = os.path.join(self.mount_directory, device)
        folder_test = os.path.join(mount_folder, test_name)

        check_existence(self.mount_directory, mount_folder)
        try:
            os.rmdir(folder_test)
        except FileNotFoundError:
            pass
        except Exception as e:
            print(e)
            print('Removal of {0} failed because the directory is not empty or still a mount point! Please unmount and remove manually: sudo umount {0}'.format(folder_test))
            return None

        if os.listdir(mount_folder):
            self.sig_info.emit('First unmount {0}'.format(device))
            return None
        else:
            pass

        devices = ['/dev/sd{0}'.format(chr(i)) for i in range(ord('a'), ord('z') + 1)]
        useable_partitions = []

        existing_devices = [entry for entry in devices if os.path.exists(entry)]

        with open('/proc/mounts', 'r') as read:
            lines = read.read()
        existing_devices = [
            entry for entry in existing_devices
            if entry not in lines
            ]

        if not existing_devices:
            self.sig_info.emit('All mountable devices are already mounted!')
            os.rmdir(mount_folder)
            return None
        else:
            pass

        existing_partitions = [
            '{0}{1}'.format(entry, number)
            for entry in existing_devices
            for number in range(10)
            if os.path.exists('{0}{1}'.format(entry, number))
            ]

        for entry in existing_partitions:
            try:
                os.mkdir(folder_test)
            except FileExistsError:
                try:
                    os.rmdir(folder_test)
                    os.mkdir(folder_test)
                except OSError:
                    self.sig_info.emit('Check folder {0} and remove it manually.'.format(folder_test))
                    return None

            cmd = "sudo -S -k mount.exfat -o uid={0} {1} {2}".format(
                os.environ['USER'],
                entry,
                folder_test
                )
            idx, value = self._start_process(cmd)

            if 'ERROR' in value or idx != 0:
                cmd = "sudo -S -k mount.ntfs {0} {1}".format(
                    entry,
                    folder_test
                    )
                idx, value = self._start_process(cmd)
                if 'ERROR' in value or idx != 0:
                    cmd = "sudo -S -k mount -o uid={0} {1} {2}".format(
                        os.environ['USER'],
                        entry,
                        folder_test
                        )
                    idx, value = self._start_process(cmd)
                else:
                    pass
            else:
                pass

            if idx == 0 and 'ERROR' not in value:
                if shutil.disk_usage(folder_test).total > 1e12:
                    useable_partitions.append(entry)
                else:
                    self.sig_info.emit(
                        '{0}: Partition smaller then 1TB'.format(entry)
                        )
                self.umount(device, test_name, '', None)
            else:
                self.sig_info.emit('{0}: Mounting error - {1}'.format(entry, value))
                if os.path.exists(folder_test):
                    os.rmdir(folder_test)
                else:
                    pass

        if useable_partitions:
            for idx, entry in enumerate(useable_partitions):
                device_name = 'HDD_{0}'.format(idx)
                folder_name = os.path.join(mount_folder, device_name)
                os.mkdir(folder_name)

                cmd = "sudo -S -k mount.exfat -o uid={0} {1} {2}".format(
                    os.environ['USER'],
                    entry,
                    folder_name
                    )
                idx, value = self._start_process(cmd)

                if 'ERROR' in value:
                    cmd = "sudo -S -k mount.ntfs {0} {1}".format(
                        entry,
                        folder_name
                        )
                    idx, value = self._start_process(cmd)
                    if 'ERROR' in value:
                        cmd = "sudo -S -k mount -o uid={0} {1} {2}".format(
                            os.environ['USER'],
                            entry,
                            folder_name
                            )
                        idx, value = self._start_process(cmd)
                    else:
                        pass
                else:
                    pass

                if idx == 0:
                    self._write_save_file(
                        user='Connected',
                        folder='',
                        mount_folder=folder_name,
                        device=device_name,
                        text='Connected',
                        folder_from_root='',
                        )
                else:
                    self.sig_error.emit('Mount error {0}: {1}'.format(
                        device_name,
                        value
                        ))
        else:
            os.rmdir(mount_folder)

        self.refresh_quota()

    @pyqtSlot(str, str, str, str, str, str, str, str, str, str, str, str)
    def mount(self, device, user, password, folder, server, typ, domain, version, sec, gid, folder_from_root, fixed_folder):
        """
        Mount device except HDD

        Arguments:
        device - Device name
        user - Username
        password - User password
        folder - Mount folder
        server - Server name
        typ - Mount type
        domain - Domain name
        version - Mount type version
        sec - security protocol
        gid - groupid to mount
        folder_from_root - Absolute path pointing towards the mount point

        Return:
        None
        """

        self.refresh_clr = 0
        self.refresh_billy = 0

        if fixed_folder:
            mount_folder = fixed_folder
            self.refresh_count[device] = 0
            self._write_save_file(
                user=user,
                folder=folder,
                mount_folder=mount_folder,
                device=device,
                text=user,
                folder_from_root=folder_from_root,
                )
        else:
            mount_folder = os.path.join(self.mount_directory, device)
            self.password_dict[device] = password

            options = ['-o nolock']
            if typ == 'cifs' or typ == 'smbfs':
                options.append("username={0},password='{1}',uid={2},vers={3},domain={4},gid={5},sec={6}".format(
                    user,
                    password,
                    os.environ['USER'],
                    version,
                    domain,
                    gid,
                    sec
                    ))
                if sec == 'krb5' or sec == 'krb5i':
                    options.append("cruid={0}".format(user))
            elif typ == 'nfs':
                pass
            else:
                print('Mountworker:', typ, ' not known! Exiting here!')
                sys.exit(1)

            cmd = "sudo -S -k mount.{0} {1} {2}/{3}/ {4}".format(
                typ,
                ','.join(options),
                server,
                folder,
                mount_folder
                )

            idx, value = self._start_process(cmd)
            if 'ERROR' in value:
                cmd = "sudo -S -k mount {0} {1}/{2}/ {3}".format(
                    ','.join(options),
                    server,
                    folder,
                    mount_folder
                    )
                idx, value = self._start_process(cmd)
            else:
                pass
            cmd = cmd.replace(password, 'PASSWORD')

            if 'mount error' in value or 'bad UNC' in value:
                print(cmd, ' - Failed:', value)
                try:
                    os.rmdir(mount_folder)
                except OSError:
                    self.sig_info.emit(
                        'Could not mount {0}: {1}'.format(mount_folder, value)
                        )
                self.sig_info.emit(
                    'Could not mount {0}: {1}'.format(mount_folder, value)
                    )
            elif idx == 0:
                print(cmd, ' - Worked:', value)
                self.refresh_count[device] = 0
                self._write_save_file(
                    user=user,
                    folder=folder,
                    mount_folder=mount_folder,
                    device=device,
                    text=user,
                    folder_from_root=folder_from_root,
                    )
            else:
                print(cmd, ' - Failed:', value)
                os.rmdir(mount_folder)
                self.sig_error.emit('Mount failed', device)

        self.refresh_quota()

    def _write_save_file(self, user, folder, mount_folder, device, text, folder_from_root):
        """
        Write a save file

        Arguments:
        user - Username
        folder - Mount folder
        mount_folder - Mount folder
        device - Device name
        text - Text

        Return:
        None
        """
        with open(self.save_files[device], 'w') as write:
            write.write('{0}\t{1}\t{2}\t{3}\t{4}\t{5}\t{6}\t{7}'.format(
                user,
                folder,
                mount_folder,
                device,
                self.ssh_dict[device],
                self.is_right_quota_dict[device],
                self.quota_dict[device],
                folder_from_root,
                ))
        self.sig_success.emit(text, device, 'lightgreen')

    @pyqtSlot(str, str, str, object)
    def umount(self, device_folder, device, fixed_folder, thread_object):
        """
        Unmount device

        Arguments:
        device_folder - Mount point folder
        device - Device name
        thread_object - Thread object that is connected to the mount point

        Return:
        None
        """

        if fixed_folder:
            self.sig_success.emit('Not connected', device, 'white')
            device = device.split('/')[-1]
            with open(self.save_files[device], 'w') as write:
                write.write('')
            self.refresh_quota()
            return None

        if 'HDD' == os.path.basename(device_folder):
            mount_folder = os.path.join(self.mount_directory, device_folder, device)
        else:
            mount_folder = os.path.join(self.mount_directory, device_folder)

        if not check_existence(self.mount_directory, mount_folder):
            try:
                os.rmdir(mount_folder)
            except OSError:
                self.sig_info.emit(
                    '\n'.join([
                        '{0} is not accessable!'.format(mount_folder),
                        'The computer might need to be restarted!',
                        'Please contact your system administrator!'
                        ])
                    )
            else:
                self.sig_info.emit('First mount {0}'.format(mount_folder))
            return None

        self.abort_finished = False
        if thread_object is None:
            pass
        elif thread_object.running:
            thread_object.kill_thread = True
            while not self.abort_finished:
                QThread.sleep(1)
            thread_object.kill_thread = False
        else:
            pass
        self.abort_finished = False

        cmd = 'sudo -S -k umount {0}'.format(mount_folder)
        idx, value = self._start_process(cmd)

        if 'mount error' in value:
            self.sig_info.emit('Could not umount {0}: {1}'.format(mount_folder, value))
        elif idx == 1:
            self.sig_info.emit('Could not umount {0}: {1}'.format(mount_folder, value))
        elif 'hdd_test' in mount_folder:
            os.rmdir(mount_folder)
        else:
            try:
                os.rmdir(mount_folder)
            except OSError:
                if self.password:
                    self.sig_info.emit(
                        'Could not umount {0}: {1}'.format(mount_folder, value.replace(self.password, 'SUDOPASSWORD'))
                        )
                else:
                    self.sig_info.emit(
                        'Could not umount {0}: {1}'.format(mount_folder, value)
                        )
                return
            device = device.split('/')[-1]
            self.sig_success.emit('Not connected', device, 'white')
            with open(self.save_files[device], 'w') as write:
                write.write('')

            if device_folder == 'HDD':
                hdd_folder = '{0}/{1}'.format(self.mount_directory, device_folder)
                if not os.listdir(hdd_folder):
                    os.rmdir(hdd_folder)

        self.refresh_quota()

    def _start_process(self, command):
        """
        Start the process with pexpect.

        Arguments:
        command - Command to run

        Return:
        index of expect, text of expect
        """
        child = pe.spawnu(command)
        child.sendline(self.password)
        try:
            idx = child.expect([pe.EOF, 'sudo: 1 incorrect password attempt'])
        except pe.exceptions.TIMEOUT:
            idx = 1
            value = 'ERROR: Do you have sudo rights for mounting?' 
        else:
            if self.password:
                if self.password:
                    value = child.before.replace(self.password, 'SUDOPASSWORD')
                else:
                    value = child.before
            else:
                value = child.before
            child.interact()

        if list(filter(lambda x: x in value, ['Error', 'Failed'])):
            idx = 1
        return idx, value

def check_existence(mount_directory, mount_folder):
    """
    Check existence of the mount folder and create it if it does not

    Arguments:
    mount_folder - tolder to check

    Return:
    True, if the mount folder exists
    """

    if not os.path.exists(mount_directory):
        try:
            os.mkdir(mount_directory)
        except OSError:
            return False
    else:
        pass

    if not os.path.exists(mount_folder):
        try:
            os.mkdir(mount_folder)
        except OSError:
            try:
                os.listdir(mount_folder)
            except OSError as err:
                if 'Required key not available:' in str(err):
                    return True
                else:
                    return False
            else:
                return False
        else:
            return False
    else:
        return True



"""
    TranSPHIRE is supposed to help with the cryo-EM data collection
    Copyright (C) 2017 Markus Stabrin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
from PyQt5.QtCore import pyqtSlot, Qt, QEvent
from PyQt5.QtWidgets import QMainWindow, QDockWidget, QTabWidget, QWidget, QHBoxLayout, QVBoxLayout, QLabel, QStyle, QPushButton


class TwinContainer(QWidget):

    def __init__(self, dock_widget, *args, parent=None, **kwargs):
        super(TwinContainer, self).__init__(parent)
        self.parent = parent
        self.layout = QVBoxLayout(self)
        self.layouts = {}
        self.dock_widget = dock_widget

    def add_to_layout(self, name, widget):
        if name not in self.layouts:
            keys = sorted(list(self.layouts) + [name])
            idx = keys.index(name)
            self.layouts[name] = QHBoxLayout()
            self.layout.insertLayout(idx, self.layouts[name])
        self.layouts[name].addWidget(widget)
        widget.mpl_canvas.sig_twin.connect(self.mouse_twin_event)

    @pyqtSlot(object)
    def mouse_twin_event(self, event):
        if event.button == 1:
            self.parent.select_tab(self.sender())
        elif event.button == 3:
            plot_widget = self.sender().my_parent
            plot_widget.trim_widget.sig_set_state.emit(True)

    def handle_show(self, name, widget, state):
        widget.setVisible(state)
        if state:
            self.layouts[name].addWidget(widget)
        else:
            self.layouts[name].removeWidget(widget)


class PlotContainer(QMainWindow):
    """
    Qwidget in combination with a FigureCanvas.

    Inherits:
    QMainWindow
    """

    def __init__(self, name, content, plot_labels, plot_name, plot_worker, plot_type, layout, *args, parent=None, **kwargs):
        """
        Initialisation of the PlotContainer widget.

        content - Content for the plotcontainer
        plot_lables - Labels of the plot widget
        plot_name - Name of the associated software
        parent - Parent widget (default None)

        Returns:
        None
        """
        super(PlotContainer, self).__init__(parent)
        self.parent_layout = layout
        self.parent = parent
        self.setCentralWidget(None)
        self.setTabPosition(Qt.TopDockWidgetArea, QTabWidget.North)
        self.plot_name = plot_name
        self.name = name

        self.worker = plot_worker

        self.content = []
        self.dock_widgets = []
        if self.name == 'Overview':
            plot_labels = [[content]]

        for label in plot_labels:
            label = label[0]
            if label == 'mic_number':
                continue
            elif label == 'file_name':
                continue
            elif label == 'image' and content != 'image':
                continue
            elif label != 'image' and content == 'image':
                continue
            elif label == 'object':
                continue
            else:
                pass

            dock_widget = QDockWidget(label, self)

            custom_title = QWidget(dock_widget)
            layout_custom_title = QHBoxLayout(custom_title)
            layout_custom_title.setContentsMargins(0, 0, 0, 0)
            layout_custom_title.addWidget(QLabel('{} - {}'.format(self.plot_name, label), dock_widget))

            layout_custom_title.addStretch(1)

            button = QPushButton(dock_widget)
            button.my_docker = dock_widget
            button.setStyleSheet('color: rgba(0, 0, 0 ,0); background-color: rgba(0, 0, 0, 0)')
            icon = dock_widget.style().standardIcon(QStyle.SP_TitleBarNormalButton)
            button.setIcon(icon)
            button.clicked.connect(self.set_floating)
            layout_custom_title.addWidget(button)

            dock_widget.setTitleBarWidget(custom_title)

            if label == 'overview':
                widget = TwinContainer(dock_widget=dock_widget, parent=self)
            else:
                if label == 'image':
                    twin_container = None
                else:
                    twin_container = self.parent.content[layout].widget(0).content[0]
                widget = PlotWidget(label=label, plot_typ=content, dock_widget=dock_widget, twin_container=twin_container, parent=self)

            self.content.append(widget)
            dock_widget.setWidget(widget)
            dock_widget.installEventFilter(self)
            dock_widget.setFeatures(QDockWidget.DockWidgetFloatable | QDockWidget.DockWidgetMovable)
            self.dock_widgets.append(dock_widget)
            self.addDockWidget(Qt.BottomDockWidgetArea, dock_widget, Qt.Horizontal)

        for idx in range(1, len(self.dock_widgets)):
            self.tabifyDockWidget(self.dock_widgets[0], self.dock_widgets[idx])
        self.tabifiedDockWidgetActivated.connect(self.synchronize_tabs)
        self.parent.content[self.parent_layout].enable_tab(False)
        self._is_visible = False

    @pyqtSlot()
    def reset_plot(self):
        for entry in self.content:
            entry.do_data_reset()

    @pyqtSlot()
    def set_floating(self):
        self.sender().my_docker.setFloating(not self.sender().my_docker.isFloating())

    @pyqtSlot(str, str, object, str, object)
    def update_figure(self, name, name_no_feedback, data, directory_name, settings):
        """
        Update the figure in the canvas

        name - Name of the program that called this function.
        data - Data to plot.
        directory_name - Directory to save plots to.
        settings - TranSPHIRE settings

        Returns:
        None
        """
        if self.plot_name == name:
            for entry in self.content:
                entry.set_settings(
                    name=name,
                    name_no_feedback=name_no_feedback,
                    data=data,
                    directory_name=directory_name,
                    settings=settings
                    )

    def eventFilter(self, source, event):
        """
        Override the QMainWindow eventFilter function.

        source - Source that led to the event trigger
        event - Emitted event

        Returns:
        True, if it has been a close event -> Redock widget
        Event, if it is another event
        """
        if event.type() == QEvent.Close and isinstance(source, QDockWidget):
            event.ignore()
            source.setFloating(False)
            return True
        else:
            return super(PlotContainer, self).eventFilter(source, event)

    def enable(self, var, use_all):
        """
        Enable or disable the widgets.

        var - It True, enable widgets, else disable
        use_all - If True, enable/disable all widgets, else only a subset

        Returns:
        None
        """
        for entry in self.content:
            if use_all:
                entry.setEnabled(var)
            else:
                pass

    def activate_tab(self, name):
        """
        Activate the tab with the name: name.

        Arguments:
        name - Name of the activation

        Returns:
        None
        """
        for widget in self.dock_widgets:
            if widget.windowTitle() == name:
                widget.show()
                widget.raise_()
                break

    @pyqtSlot(bool, str)
    def set_visibility(self, visible, name):
        if name == self.plot_name:
            if self._is_visible != visible:
                self.parent.content[self.parent_layout].enable_tab(visible)
                if visible:
                    for widget in self.content:
                        widget.show()
                        widget.start_plotting()
                self._is_visible = visible

    @pyqtSlot(object)
    def select_tab(self, widget):
        aim_docker = self.parent.content[self.parent_layout]
        cur_docker = aim_docker
        for _ in range(3):
            tmp_docker = self.parent.content[cur_docker.layout]
            tmp_docker.setCurrentWidget(cur_docker)
            cur_docker = tmp_docker

        docker_to_activate = None
        for idx in range(aim_docker.count()):
            aim_container = aim_docker.widget(idx)
            for entry in aim_container.dock_widgets:
                if entry.widget() == widget.my_parent:
                    docker_to_activate = entry
                    break
            else:
                continue
            break
        assert docker_to_activate is not None, (entry, widget, widget.my_parent)
        self.synchronize_tabs(docker_to_activate)
        aim_docker.setCurrentIndex(idx)

    @pyqtSlot(QDockWidget)
    def synchronize_tabs(self, widget):
        compare_name = widget.windowTitle()
        aim_docker = self.parent.content[self.parent_layout]
        aim_indices = []
        for idx in range(aim_docker.count()):
            tab_text = aim_docker.tabText(idx)
            if self.name == 'Overview':
                if tab_text == 'Plot histogram':
                    aim_indices.append(idx)
                    continue
                elif tab_text == 'Plot per micrograph':
                    aim_indices.append(idx)
                    continue
            elif self.name == 'Plot per micrograph':
                if tab_text == 'Plot histogram':
                    aim_indices.append(idx)
                    break
            elif self.name == 'Plot histogram':
                if tab_text == 'Plot per micrograph':
                    aim_indices.append(idx)
                    break
            else:
                pass

        assert aim_indices is not None
        for aim_idx in aim_indices:
            aim_container = aim_docker.widget(aim_idx)
            aim_container.activate_tab(compare_name)

from .plotwidget import PlotWidget



"""
    TranSPHIRE is supposed to help with the cryo-EM data collection
    Copyright (C) 2017 Markus Stabrin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import json
import glob
import os
import shutil
import numpy as np
import mrcfile as mrc
import matplotlib
matplotlib.use('QT5Agg')
import matplotlib.image as mi

from . import transphire_import as ti
from . import transphire_utils as tu


def get_ctf_command(file_sum, file_input, new_name, settings, queue_com, set_name, name):
    """
    Create the ctf command based on the ctf software.

    file_input - Input name of the file for ctf estimation
    new_name - Output file
    settings - TranSPHIRE settings
    queue_com - Queue for communication
    name - Name of process

    Returns:
    CTF command
    File to check vor validation if the process was successful
    """
    ctf_name = settings['Copy']['CTF']
    command = None
    block_gpu = None
    gpu_list = None
    shell = None
    if 'CTFFIND4' in ctf_name:
        if tu.is_higher_version(ctf_name, '4.1.13'):
            command = create_ctffind_4_v4_1_13_command(
                ctf_name = ctf_name,
                file_sum=file_sum,
                file_input=file_input,
                file_output=new_name,
                set_name=set_name,
                settings=settings
                )
            check_files = []
            block_gpu = False
            gpu_list = []
            shell = True
        elif tu.is_higher_version(ctf_name, '4.1.8'):
            command = create_ctffind_4_v4_1_8_command(
                ctf_name = ctf_name,
                file_sum=file_sum,
                file_input=file_input,
                file_output=new_name,
                set_name=set_name,
                settings=settings
                )
            check_files = []
            block_gpu = False
            gpu_list = []
            shell = True

    elif 'Gctf' in ctf_name:
        if tu.is_higher_version(ctf_name, '1.06'):
            command, gpu = create_gctf_v1_06_command(
                ctf_name=ctf_name,
                file_sum=file_sum,
                file_input=file_input,
                file_output=new_name,
                settings=settings,
                name=name
                )
            check_files = ['{0}_gctf.star'.format(new_name)]
            if tu.is_higher_version(ctf_name, '1.18'):
                block_gpu = True
            elif tu.is_higher_version(ctf_name, '1.06'):
                block_gpu = True
            else:
                assert False
            gpu_list = gpu.split()
            shell = False

    elif 'CTER' in ctf_name:
        if tu.is_higher_version(ctf_name, '1.3'):
            output_dir, _ = os.path.splitext(new_name)
            command = create_cter_1_3_command(
                ctf_name=ctf_name,
                file_sum=file_sum,
                file_input=file_input,
                output_dir=output_dir,
                settings=settings
                )
            check_files = ['{0}/partres.txt'.format(output_dir)]
            block_gpu = False
            gpu_list = []
            shell = True
        elif tu.is_higher_version(ctf_name, '1.0'):
            output_dir, _ = os.path.splitext(new_name)
            command = create_cter_1_0_command(
                ctf_name=ctf_name,
                file_sum=file_sum,
                file_input=file_input,
                output_dir=output_dir,
                settings=settings
                )
            check_files = ['{0}/partres.txt'.format(output_dir)]
            block_gpu = False
            gpu_list = []
            shell = False

    else:
        message = '\n'.join([
            '{0}: Not known!'.format(settings['Copy']['CTF']),
            'Please contact the TranSPHIRE authors!'
            ])
        queue_com['error'].put(
            message,
            name
            )
        IOError(message)

    assert command is not None, 'command not specified: {0}'.format(ctf_name)
    assert block_gpu is not None, 'block_gpu not specified: {0}'.format(ctf_name)
    assert gpu_list is not None, 'gpu_list not specified: {0}'.format(ctf_name)
    assert shell is not None, 'shell not specified: {0}'.format(ctf_name)

    return command, check_files, block_gpu, gpu_list, shell


def find_logfiles(root_path, file_name, settings, queue_com, name):
    """
    Find logfiles related to the produced CTF files.

    root_path - Root path of the file
    file_name - File name of the ctf file.
    settings - TranSPHIRE settings
    queue_com - Queue for communication
    name - Name of process

    Returns:
    list of log files
    """
    log_files = None
    copied_log_files = None
    ctf_root_path = os.path.join(settings['ctf_folder_feedback_0'], file_name)
    ctf_name = settings['Copy']['CTF']
    if 'CTFFIND4' in ctf_name:
        if tu.is_higher_version(ctf_name, '4.1.8'):
            copied_log_files = []
            recursive_file_search(directory=ctf_root_path, files=copied_log_files)
            log_files = copied_log_files

    elif 'CTER' in ctf_name:
        if tu.is_higher_version(ctf_name, '1'):
            copied_log_files = []
            recursive_file_search(directory=ctf_root_path, files=copied_log_files)
            log_files = copied_log_files

    elif 'Gctf' in ctf_name:
        if tu.is_higher_version(ctf_name, '1.06'):
            log_files = []
            copied_log_files = []
            exclude_list = [
                '{0}.mrc'.format(root_path),
                '{0}_DW.mrc'.format(root_path),
                '{0}.log'.format(root_path),
                '{0}.err'.format(root_path),
                ]
            for log_file in glob.glob('{0}*'.format(root_path)):
                if log_file in exclude_list:
                    continue
                else:
                    pass

                new_file = os.path.join(
                    settings['ctf_folder_feedback_0'],
                    os.path.basename(log_file)
                    )
                log_files.append(log_file)
                copied_log_files.append(new_file)

    else:
        message = '\n'.join([
            '{0}: Not known!'.format(settings['Copy']['CTF']),
            'Please contact the TranSPHIRE authors!'
            ])
        queue_com['error'].put(
            message,
            name
            )
        raise IOError(message)

    assert log_files is not None
    assert copied_log_files is not None
    return log_files, copied_log_files


def recursive_file_search(directory, files):
    """
    Recursive file search function.
    """
    file_names = glob.glob('{0}*'.format(directory))
    for name in file_names:
        if os.path.isdir(name):
            recursive_file_search('{0}/'.format(name), files)
        else:
            files.append(name)


def create_gctf_v1_06_command(
        ctf_name, file_sum, file_input, file_output, settings, name
        ):
    """Create the Gctf v1.06 command"""

    ignore_list = []
    ignore_list.append('Use movies')
    ignore_list.append('Phase plate')
    command = []
    # Start the program
    command.append('{0}'.format(settings['Path'][ctf_name]))
    if settings[ctf_name]['Phase plate'] == 'False':
        ignore_list.append('--phase_shift_L')
        ignore_list.append('--phase_shift_H')
        ignore_list.append('--phase_shift_S')
        ignore_list.append('--phase_shift_T')
        ignore_list.append('--only_search_ps')
        ignore_list.append('--cosearch_refine_ps')
    else:
        pass
    if settings[ctf_name]['Use movies'] == 'False':
        ignore_list.append('--do_mdef_refine')
        ignore_list.append('--mdef_ave_type')
        ignore_list.append('--mdef_aveN')
        ignore_list.append('--mdef_fit')
        file_input = file_sum
    else:
        pass

    ignore_list.append('Split Gpu?')
    ignore_list.append('--gid')
    if settings[ctf_name]['Split Gpu?'] == 'True':
        try:
            gpu_id = int(name.split('_')[-1])
        except ValueError:
            gpu_id = 0
        try:
            gpu = settings[ctf_name]['--gid'].split()[gpu_id]
        except IndexError:
            raise UserWarning('There are less gpus provided than threads available! Please restart with the same number of pipeline processors as GPUs provided and restart! Stopping this thread!')
    else:
        gpu = settings[ctf_name]['--gid']

    command.append('--gid')
    command.append('{0}'.format(gpu))

    command.append('--ctfstar')
    command.append('{0}_gctf.star'.format(file_output))

    for key in settings[ctf_name]:
        if key in ignore_list:
            continue
        elif settings[ctf_name][key]:
            command.append(key)
            command.append(
                '{0}'.format(settings[ctf_name][key])
                )
        else:
            continue

    command.append('{0}'.format(file_input))

    return ' '.join(command), gpu


def create_cter_1_3_command(
        ctf_name, file_sum, file_input, output_dir, settings
        ):
    """Create the CTER v1.3 command"""

    try:
        shutil.rmtree(output_dir)
    except FileNotFoundError:
        pass

    command = []
    command.append("PATH=$(dirname $(which {0})):${{PATH}}".format(settings['Path'][ctf_name]))
    # Start the program
    command.append('{0}'.format(settings['Path'][ctf_name]))
    command.append("'{0}*'".format(file_sum[:-1]))
    command.append(output_dir)
    command.append('--selection_list={0}'.format(file_sum))
    ignore_list = []
    ignore_list.append('Phase plate')
    if settings[ctf_name]['Phase plate'] == 'False':
        ignore_list.append('--defocus_min')
        ignore_list.append('--defocus_max')
        ignore_list.append('--defocus_step')
        ignore_list.append('--phase_min')
        ignore_list.append('--phase_max')
        ignore_list.append('--phase_step')
    else:
        command.append('--vpp')

    ignore_list.append('--pap')
    if settings[ctf_name]['--pap'] == 'True':
        command.append('--pap')
    else:
        pass

    for key in settings[ctf_name]:
        if key in ignore_list:
            continue
        elif settings[ctf_name][key]:
            command.append('{0}={1}'.format(key, settings[ctf_name][key]))
        else:
            continue

    # Prepare HDF
    command.append(';')
    command.append("PATH=$(dirname $(which {0})):${{PATH}}".format(settings['Path']['e2proc2d.py']))
    command.append(settings['Path']['e2proc2d.py'])
    command.append(os.path.join(output_dir, 'power2d', '{0}_pws.hdf'.format(tu.get_name(file_sum))))
    command.append(os.path.join(output_dir, 'power2d', '{0}_pws.mrc'.format(tu.get_name(file_sum))))

    return ' '.join(command)

def create_cter_1_0_command(
        ctf_name, file_sum, file_input, output_dir, settings
        ):
    """Create the CTER v1.0 command"""

    try:
        shutil.rmtree(output_dir)
    except FileNotFoundError:
        pass

    command = []
    # Start the program
    command.append('{0}'.format(settings['Path'][ctf_name]))
    command.append("{0}*".format(file_sum[:-1]))
    command.append(output_dir)
    command.append('--selection_list={0}'.format(file_sum))
    ignore_list = []
    ignore_list.append('Phase plate')
    if settings[ctf_name]['Phase plate'] == 'False':
        ignore_list.append('--defocus_min')
        ignore_list.append('--defocus_max')
        ignore_list.append('--defocus_step')
        ignore_list.append('--phase_min')
        ignore_list.append('--phase_max')
        ignore_list.append('--phase_step')
    else:
        command.append('--vpp')

    ignore_list.append('--pap')
    if settings[ctf_name]['--pap'] == 'True':
        command.append('--pap')
    else:
        pass

    for key in settings[ctf_name]:
        if key in ignore_list:
            continue
        elif settings[ctf_name][key]:
            command.append('{0}={1}'.format(key, settings[ctf_name][key]))
        else:
            continue

    return ' '.join(command)

def create_ctffind_4_v4_1_13_command(ctf_name, file_sum, file_input, file_output, set_name, settings):
    """Create the ctffind command"""
    ctf_settings = settings[ctf_name]
    ctffind_command = []
    if ctf_settings['Use movies'] == 'True':
        pass
    else:
        file_input = file_sum
    # Input file
    ctffind_command.append('{0}'.format(file_input))
    # Is movie
    if ctf_settings['Use movies'] == 'True':
        ctffind_command.append('{0}'.format('yes'))
        ctffind_command.append('{0}'.format(ctf_settings['Combine frames']))
    else:
        pass
    # Output file
    ctffind_command.append('{0}'.format(file_output))
    # Pixel size
    ctffind_command.append('{0}'.format(ctf_settings['Pixel size']))
    # Acceleration voltage
    ctffind_command.append('{0}'.format(ctf_settings['Acceleration voltage']))
    # Spherical abberation
    ctffind_command.append('{0}'.format(ctf_settings['Spherical aberration']))
    # Amplitude contrast
    ctffind_command.append('{0}'.format(ctf_settings['Amplitude contrast']))
    # Amplitude spectrum
    ctffind_command.append('{0}'.format(ctf_settings['Amplitude spectrum']))
    # Minimum resolution
    ctffind_command.append('{0}'.format(ctf_settings['Min resolution(A)']))
    # Maximum resolution
    ctffind_command.append('{0}'.format(ctf_settings['Max resolution(A)']))
    # Minimum defocus
    ctffind_command.append('{0}'.format(ctf_settings['Min defocus(A)']))
    # Maximum defocus
    ctffind_command.append('{0}'.format(ctf_settings['Max defocus(A)']))
    # Defocus step
    ctffind_command.append('{0}'.format(ctf_settings['Step defocus(A)']))
    # Do you know astigmatism
    if ctf_settings['Know astigmatism'] == 'True':
        ctffind_command.append('{0}'.format('yes'))
    else:
        ctffind_command.append('{0}'.format('no'))
    # Slow or fast
    if ctf_settings['High accuracy'] == 'True':
        ctffind_command.append('{0}'.format('yes'))
    else:
        ctffind_command.append('{0}'.format('no'))
    # Astigmatism ctf_settings
    if ctf_settings['Know astigmatism'] == 'True':
        ctffind_command.append('{0}'.format(ctf_settings['Astigmatism']))
        ctffind_command.append('{0}'.format(ctf_settings['Astigmatism angle']))
    else:
        if ctf_settings['Restrain astigmatism'] == 'True':
            ctffind_command.append('{0}'.format('yes'))
            ctffind_command.append(
                '{0}'.format(ctf_settings['Expected astigmatism'])
                )
        else:
            ctffind_command.append('{0}'.format('no'))
    # Phase shift
    if ctf_settings['Phase shift'] == 'True':
        ctffind_command.append('{0}'.format('yes'))
        ctffind_command.append('{0}'.format(ctf_settings['Min phase(rad)']))
        ctffind_command.append('{0}'.format(ctf_settings['Max phase(rad)']))
        ctffind_command.append('{0}'.format(ctf_settings['Step phase(rad)']))
    else:
        ctffind_command.append('{0}'.format('no'))
    # Expert ctf_settings
    ctffind_command.append('{0}'.format('yes'))
    # Resmaple micrograph
    if ctf_settings['Resample micrographs'] == 'True':
        ctffind_command.append('{0}'.format('yes'))
    else:
        ctffind_command.append('{0}'.format('no'))
    # Movie input ctf_settings
    if ctf_settings['Use movies'] == 'True':
        # Gain correction
        if ctf_settings['Movie is gain-corrected?'] == 'True':
            ctffind_command.append('{0}'.format('yes'))
        else:
            ctffind_command.append('{0}'.format('no'))
            external_log, key = ctf_settings['Gain file'].split('|||')
            with open(settings[external_log], 'r') as read:
                log_data = json.load(read)
            gain_file = log_data[set_name][ctf_name][key]['new_file']
            ctffind_command.append(gain_file)

        # Correct for magnifying distortions
        if ctf_settings['Correct mag. distort.'] == 'True':
            ctffind_command.append('{0}'.format('yes'))
            ctffind_command.append(
                '{0}'.format(ctf_settings['Mag. dist. angle'])
                )
            ctffind_command.append(
                '{0}'.format(ctf_settings['Mag. dist. major scale'])
                )
            ctffind_command.append(
                '{0}'.format(ctf_settings['Mag. dist. minor scale'])
                )
        else:
            ctffind_command.append('{0}'.format('no'))

    else:
        pass
    ctffind_command.append('{0}'.format('no'))
    ctffind_command.append('{0}'.format(1))

    command = 'echo "{0}" | {1}'.format(
        '\n'.join(ctffind_command),
        settings['Path'][ctf_name]
        )
    return command


def create_ctffind_4_v4_1_8_command(ctf_name, file_sum, file_input, file_output, set_name, settings):
    """Create the ctffind command"""
    ctf_settings = settings[ctf_name]
    ctffind_command = []
    if ctf_settings['Use movies'] == 'True':
        pass
    else:
        file_input = file_sum
    # Input file
    ctffind_command.append('{0}'.format(file_input))
    # Is movie
    if ctf_settings['Use movies'] == 'True':
        ctffind_command.append('{0}'.format('yes'))
        ctffind_command.append('{0}'.format(ctf_settings['Combine frames']))
    else:
        pass
    # Output file
    ctffind_command.append('{0}'.format(file_output))
    # Pixel size
    ctffind_command.append('{0}'.format(ctf_settings['Pixel size']))
    # Acceleration voltage
    ctffind_command.append('{0}'.format(ctf_settings['Acceleration voltage']))
    # Spherical abberation
    ctffind_command.append('{0}'.format(ctf_settings['Spherical aberration']))
    # Amplitude contrast
    ctffind_command.append('{0}'.format(ctf_settings['Amplitude contrast']))
    # Amplitude spectrum
    ctffind_command.append('{0}'.format(ctf_settings['Amplitude spectrum']))
    # Minimum resolution
    ctffind_command.append('{0}'.format(ctf_settings['Min resolution(A)']))
    # Maximum resolution
    ctffind_command.append('{0}'.format(ctf_settings['Max resolution(A)']))
    # Minimum defocus
    ctffind_command.append('{0}'.format(ctf_settings['Min defocus(A)']))
    # Maximum defocus
    ctffind_command.append('{0}'.format(ctf_settings['Max defocus(A)']))
    # Defocus step
    ctffind_command.append('{0}'.format(ctf_settings['Step defocus(A)']))
    # Do you know astigmatism
    if ctf_settings['Know astigmatism'] == 'True':
        ctffind_command.append('{0}'.format('yes'))
    else:
        ctffind_command.append('{0}'.format('no'))
    # Slow or fast
    if ctf_settings['High accuracy'] == 'True':
        ctffind_command.append('{0}'.format('yes'))
    else:
        ctffind_command.append('{0}'.format('no'))
    # Astigmatism ctf_settings
    if ctf_settings['Know astigmatism'] == 'True':
        ctffind_command.append('{0}'.format(ctf_settings['Astigmatism']))
        ctffind_command.append('{0}'.format(ctf_settings['Astigmatism angle']))
    else:
        if ctf_settings['Restrain astigmatism'] == 'True':
            ctffind_command.append('{0}'.format('yes'))
            ctffind_command.append(
                '{0}'.format(ctf_settings['Expected astigmatism'])
                )
        else:
            ctffind_command.append('{0}'.format('no'))
    # Phase shift
    if ctf_settings['Phase shift'] == 'True':
        ctffind_command.append('{0}'.format('yes'))
        ctffind_command.append('{0}'.format(ctf_settings['Min phase(rad)']))
        ctffind_command.append('{0}'.format(ctf_settings['Max phase(rad)']))
        ctffind_command.append('{0}'.format(ctf_settings['Step phase(rad)']))
    else:
        ctffind_command.append('{0}'.format('no'))
    # Expert ctf_settings
    ctffind_command.append('{0}'.format('yes'))
    # Resmaple micrograph
    if ctf_settings['Resample micrographs'] == 'True':
        ctffind_command.append('{0}'.format('yes'))
    else:
        ctffind_command.append('{0}'.format('no'))
    # Movie input ctf_settings
    if ctf_settings['Use movies'] == 'True':
        # Gain correction
        if ctf_settings['Movie is gain-corrected?'] == 'True':
            ctffind_command.append('{0}'.format('yes'))
        else:
            ctffind_command.append('{0}'.format('no'))
            external_log, key = ctf_settings['Gain file'].split('|||')
            with open(settings[external_log], 'r') as read:
                log_data = json.load(read)
            gain_file = log_data[set_name][ctf_name][key]['new_file']
            ctffind_command.append(gain_file)

        # Correct for magnifying distortions
        if ctf_settings['Correct mag. distort.'] == 'True':
            ctffind_command.append('{0}'.format('yes'))
            ctffind_command.append(
                '{0}'.format(ctf_settings['Mag. dist. angle'])
                )
            ctffind_command.append(
                '{0}'.format(ctf_settings['Mag. dist. major scale'])
                )
            ctffind_command.append(
                '{0}'.format(ctf_settings['Mag. dist. minor scale'])
                )
        else:
            ctffind_command.append('{0}'.format('no'))

    else:
        pass

    command = 'echo "{0}" | {1}'.format(
        '\n'.join(ctffind_command),
        settings['Path'][ctf_name]
        )
    return command


def combine_ctf_outputs(
        data,
        data_orig,
        root_path,
        file_name,
        settings,
        queue_com,
        shared_dict,
        name,
        sum_file,
        dw_file,
        ):
    """
    Combine the ctf outputs to one SPHIRE partres and one RELION star file.

    root_path - Root path of the file
    file_name - File name of the ctf file.
    settings - TranSPHIRE settings
    queue_com - Queue for communication
    name - Name of process
    sum_file - Name of the dose uncorrected sum file

    Returns:
    None
    """

    ctf_name = settings['Copy']['CTF']
    ctf_settings = settings[ctf_name]
    ctf_folder = settings['ctf_folder_feedback_0']
    project_folder = '{0}/'.format(settings['project_folder'].rstrip('/'))
    project_base = '{0}/'.format(settings['project_base'].rstrip('/'))

    try:
        if ctf_settings['Use movies'] == 'True':
            motion_name = settings['Copy']['Motion']
            pixel_size_adjust = float(settings[motion_name]['-FtBin'])
        else:
            pixel_size_adjust = 1
    except KeyError:
        pixel_size_adjust = 1


    if ctf_name.lower().startswith('cter'):
        data_star = data
    else:
        data_star = data_orig

    lines = to_star_file(
        data=np.copy(data_star),
        ctf_name=ctf_name,
        ctf_settings=ctf_settings,
        project_folder=project_base,
        ctf_folder=ctf_folder,
        sum_file=sum_file,
        dw_file=dw_file,
        pixel_size_adjust=pixel_size_adjust,
        )
    output_name_star = os.path.join(
        ctf_folder,
        '{0}_transphire_ctf.star'.format(os.path.basename(sum_file))
        )

    with open(output_name_star, 'w') as write:
        write.write(lines)

    lines = to_partres_file(
        data=np.copy(data_orig),
        ctf_name=ctf_name,
        ctf_settings=ctf_settings,
        project_folder=project_base,
        ctf_folder=ctf_folder,
        sum_file=sum_file,
        pixel_size_adjust=pixel_size_adjust,
        )
    output_name_partres = os.path.join(
        ctf_folder,
        '{0}_transphire_ctf_partres.txt'.format(os.path.basename(sum_file))
        )

    with open(output_name_partres, 'w') as write:
        write.write(lines)

    output_name_partres_combined = os.path.join(
        project_folder,
        '{0}_transphire_ctf_partres.txt'.format(ctf_name.replace(' ', '_').replace('>=', ''))
        )
    output_name_star_combined = os.path.join(
        project_folder,
        '{0}_transphire_ctf.star'.format(ctf_name.replace(' ', '_').replace('>=', ''))
        )
    return output_name_partres_combined, output_name_star_combined, output_name_partres, output_name_star


def to_star_file(data, ctf_name, ctf_settings, project_folder, ctf_folder, sum_file, dw_file, pixel_size_adjust):
    """
    Create a CTF star file from data

    data - Array containing ctf information.
    ctf_name - Name of the ctf program.
    ctf_settings - Settings for this ctf estimation run.
    project_folder - Name of the project folder.
    ctf_folder - Name of the ctf output folder.
    sum_file - Name of the sum file

    Returns:
    None
    """
    export_dtype = data.dtype.descr
    extension_dtype = [
        ('_rlnCtfImage', '|U1200'),
        ('_rlnVoltage', '<f8'),
        ('_rlnSphericalAberration', '<f8'),
        ('_rlnAmplitudeContrast', '<f8'),
        ('_rlnMagnification', '<f8'),
        ('_rlnDetectorPixelSize', '<f8'),
        ]

    export_dtype.extend(extension_dtype)
    if dw_file == 'None':
        pass
    else:
        export_dtype.extend([('_rlnMicrographNameNoDW', '|U1200')])
    export_data = np.atleast_1d(np.empty(data.shape[0], dtype=export_dtype))
    header = get_relion_header(names=export_data.dtype.names)

    for idx, row in enumerate(data):

        for name in row.dtype.names:

            if name == 'file_name':
                if dw_file == 'None':
                    export_data[idx]['file_name'] = sum_file.replace(project_folder, '')
                else:
                    export_data[idx]['_rlnMicrographNameNoDW'] = sum_file.replace(project_folder, '')
                    export_data[idx]['file_name'] = dw_file.replace(project_folder, '')
                continue

            elif name == 'defocus':
                value = (-1 * row['defocus_diff'] + 2 * row['defocus']) / 2
            elif name == 'defocus_diff':
                value = 2 * row['defocus'] - export_data[idx]['defocus']
            else:
                value = row[name]
            export_data[idx][name] = value

        for key, _ in extension_dtype:
            value = get_constant_value(key, ctf_settings, row, project_folder, ctf_name, ctf_folder, pixel_size_adjust)
            export_data[idx][key] = value

    lines = [header]
    maximum_string = {
        '_rlnCtfImage': len(max(export_data['_rlnCtfImage'], key=len)),
        'file_name': len(max(export_data['file_name'], key=len))
        }
    if dw_file == 'None':
        pass
    else:
        maximum_string['_rlnMicrographNameNoDW'] = len(max(export_data['_rlnMicrographNameNoDW'], key=len))
    create_export_data(
        export_data=export_data,
        lines=lines,
        maximum_string=maximum_string
        )
    return ''.join(lines)


def create_export_data(export_data, lines, maximum_string):
    """
    Write export data to file.

    export_data - Data to export.
    file_name - Name of the file to write to.

    Returns:
    In place modificaion of lines
    """
    for row in export_data:
        row_string = []
        for name in export_data.dtype.names:
            if name == 'mic_number':
                continue
            elif name == 'image':
                continue
            else:
                pass
            value = row[name]
            if isinstance(value, int):
                row_string.append('{0: 7d}'.format(value))
            elif isinstance(value, float):
                row_string.append('{0: 14f}'.format(value))
            else:
                length = maximum_string[name]
                row_string.append('{0:{1}s}'.format(value, length))
        lines.append('{0}\n'.format(' '.join(row_string)))


def to_partres_file(data, ctf_name, ctf_settings, project_folder, ctf_folder, sum_file, pixel_size_adjust):
    """
    Create a CTF partres file from data

    data - Array containing ctf information.
    ctf_name - Name of the ctf program.
    ctf_settings - Settings for this ctf estimation run.
    project_folder - Name of the project folder.
    ctf_folder - Name of the ctf output folder.
    sum_file - Name of the summed micrograph image

    Returns:
    None
    """


    dtype_import_dict_name = tu.find_latest_version('CTER', ti.get_dtype_import_dict())
    export_dtype = ti.get_dtype_import_dict()[dtype_import_dict_name]
    export_data = np.atleast_1d(np.empty(data.shape[0], dtype=export_dtype))
    constant_settings = set([
        'cs',
        'volt',
        'apix',
        'const_amplitude_contrast',
        ])

    for idx, row in enumerate(data):

        for name in export_data.dtype.names:

            try:
                value = row[name] 
            except ValueError:
                # Is not a CTER partres name
                assert name != 'phase_shift'
                assert name != 'file_name'
                if name in constant_settings:
                    value = get_constant_value(name, ctf_settings, row, project_folder, ctf_name, ctf_folder, pixel_size_adjust)
                    if name == 'const_amplitude_contrast':
                        value *= 100
                    else:
                        pass
                elif name == 'reserved_spot':
                    value = row['cross_corr']
                elif name == 'defocus':
                    value = (row['defocus_1'] + row['defocus_2']) / 20000
                elif name == 'astigmatism_amplitude':
                    value = (-row['defocus_1'] + row['defocus_2']) / 10000
                elif name == 'astigmatism_angle':
                    value = 45 - row['astigmatism']
                elif name == 'limit_defocus_and_astigmatism':
                    if row['limit'] == 0:
                        value = -1
                    else:
                        value = 1 / row['limit']
                elif name == 'limit_pixel_error':
                    value = 1 / (get_constant_value('apix', ctf_settings, row, project_folder, ctf_name, ctf_folder, pixel_size_adjust) * 2)
                elif name == 'amplitude_contrast':
                    contrast = get_constant_value('const_amplitude_contrast', ctf_settings, row, project_folder, ctf_name, ctf_folder, pixel_size_adjust) * 100
                    contrast = contrast_to_shift(contrast)
                    phase_shift = row['phase_shift']
                    value = shift_to_contrast(contrast + phase_shift)
                else:
                    value = 0
            else:
                # Is a CTER partres name
                if name == 'file_name':
                    value = '{0}.mrc'.format(tu.get_name(sum_file))
                else:
                    pass

            export_data[idx][name] = value

    lines = []
    maximum_string = {
        'file_name': len(max(export_data['file_name'], key=len))
        }
    create_export_data(
        export_data=export_data,
        lines=lines,
        maximum_string=maximum_string
        )
    return ''.join(lines)


def get_relion_header(names):
    """
    Create a relion star file header.

    names - Header names as list

    Returns:
    header string
    """
    header = []
    header.append('')
    header.append('data_')
    header.append('')
    header.append('loop_')
    transphire_dict = ti.get_transphire_dict()
    index = 0
    for name in names:
        try:
            new_name = transphire_dict[name]
        except KeyError:
            if name == 'mic_number':
                continue
            elif name == 'image':
                continue
            else:
                new_name = name
        header.append('{0} #{1}'.format(new_name, index+1))
        index += 1
    return '{0}\n'.format('\n'.join(header))


def get_constant_value(key, ctf_settings, row, project_folder, ctf_name, ctf_folder, pixel_size_adjust):
    if key == '_rlnCtfImage':
        _, extension = os.path.splitext(row['file_name'])
        if ctf_name.lower().startswith('gctf'):
            file_name = row['file_name'].replace(
                extension,
                '.ctf:mrc'
                )
            file_name = os.path.join(ctf_folder, os.path.basename(file_name))
        elif ctf_name.lower().startswith('ctffind'):
            file_name = row['file_name'].replace(
                extension,
                '.mrc:mrc'
                )
            file_name = os.path.join(ctf_folder, os.path.basename(file_name))
        elif ctf_name.lower().startswith('cter'):
            file_name = 'not_available_with_cter_option'
        else:
            raise IOError('Cannot find ctf_name! {0}'.format(
                ctf_name
                ))
        value = file_name.replace(project_folder, '')
    elif key == '_rlnVoltage' or key == 'volt':
        if '--kV' in ctf_settings:
            value = ctf_settings['--kV']
        elif 'Acceleration voltage' in ctf_settings:
            value = ctf_settings['Acceleration voltage']
        elif '--voltage' in ctf_settings:
            value = ctf_settings['--voltage']
        else:
            raise IOError('Cannot find voltage key! {0}'.format(
                ctf_name
                ))
        value = float(value)
    elif key == '_rlnSphericalAberration' or key == 'cs':
        if 'Spherical aberration' in ctf_settings:
            value = ctf_settings['Spherical aberration']
        elif '--cs' in ctf_settings:
            value = ctf_settings['--cs']
        elif '--Cs' in ctf_settings:
            value = ctf_settings['--Cs']
        else:
            raise IOError('Cannot find cs key! {0}'.format(
                ctf_name
                ))
        value = float(value)
    elif key == '_rlnAmplitudeContrast' or key == 'const_amplitude_contrast':
        if '--ac' in ctf_settings:
            value = ctf_settings['--ac']
            value = float(value)
            if value > 1:
                value /= 100
            else:
                pass
        elif 'Amplitude contrast' in ctf_settings:
            value = ctf_settings['Amplitude contrast']
        else:
            raise IOError('Cannot find ac key! {0}'.format(
                ctf_name
                ))
        value = float(value)
    elif key == '_rlnMagnification':
        value = 10000.0
        value = float(value)
    elif key == '_rlnDetectorPixelSize' or key == 'apix':
        if 'Pixel size' in ctf_settings:
            value = ctf_settings['Pixel size']
        elif '--apix' in ctf_settings:
            value = ctf_settings['--apix']
        else:
            raise IOError('Cannot find apix key! {0}'.format(
                ctf_name
                ))
        value = float(value) * pixel_size_adjust
    else:
        raise IOError('Dont know {0}'.format(
            key
            ))
    return value


def contrast_to_shift(a_cont):
    """
    Transform amplitude contrast to phase shift.

    a_cont - Amplitude contrast value in percent

    Returns:
    Phase shift in degrees.
    """
    a_cont = float(a_cont)
    if a_cont == 100.0:
        value = 90.0
    elif a_cont == -100.0:
        value = 90.0
    elif a_cont < 0.0:
        value =  np.degrees(np.arctan(a_cont / np.sqrt(1.0e4 - a_cont**2))) + 180.0
    else:
        value = np.degrees(np.arctan(a_cont / np.sqrt(1.0e4 - a_cont**2)))
    return value


def shift_to_contrast(phase_shift):
    """
    Convert phase shift to amplitud contrast.

    phase_shift - Phase shift in degrees.

    Returns:
    Amplitude contrast in percent.
    """
    return np.tan(np.radians(phase_shift)) / \
        np.sqrt(1.0 + np.tan(np.radians(phase_shift))**2) * 100.0

@tu.rerun_function_in_case_of_error
def create_jpg_file(input_mrc_file, settings, ctf_name):
    file_name = tu.get_name(input_mrc_file)
    input_ctf_file = None
    input_1d_file = None
    if 'CTFFIND4' in ctf_name:
        input_ctf_file = os.path.join(settings['ctf_folder_feedback_0'], '{0}.mrc'.format(file_name))
        input_1d_file = os.path.join(settings['ctf_folder_feedback_0'], '{0}_avrot.txt'.format(file_name))
    elif 'Gctf' in ctf_name:
        input_ctf_file = os.path.join(settings['ctf_folder_feedback_0'], '{0}.ctf'.format(file_name))
        input_1d_file = os.path.join(settings['ctf_folder_feedback_0'], '{0}_EPA.log'.format(file_name))
    elif 'CTER' in ctf_name:
        if tu.is_higher_version(ctf_name, '1.3'):
            input_ctf_file = os.path.join(settings['ctf_folder_feedback_0'], file_name, 'power2d', '{0}_pws.mrc'.format(file_name))
            input_1d_file = os.path.join(settings['ctf_folder_feedback_0'], file_name, 'pwrot', '{0}_rotinf.txt'.format(file_name))
        elif tu.is_higher_version(ctf_name, '1.0'):
            input_1d_file = os.path.join(settings['ctf_folder_feedback_0'], file_name, 'pwrot', '{0}_rotinf.txt'.format(file_name))

    try:
        if not os.path.exists(input_ctf_file):
            input_ctf_file = None
    except TypeError:
            input_ctf_file = None
    try:
        if not os.path.exists(input_1d_file):
            input_1d_file = None
    except TypeError:
        input_1d_file = None

    tu.mkdir_p(os.path.join(settings['ctf_folder_feedback_0'], 'jpg'))
    tu.mkdir_p(os.path.join(settings['ctf_folder_feedback_0'], 'jpg_2'))
    tu.mkdir_p(os.path.join(settings['ctf_folder_feedback_0'], 'json'))

    jpg_file_1 = os.path.join(settings['ctf_folder_feedback_0'], 'jpg', '{0}.jpg'.format(file_name))
    jpg_file_2 = os.path.join(settings['ctf_folder_feedback_0'], 'jpg_2', '{0}.jpg'.format(file_name))
    json_file = os.path.join(settings['ctf_folder_feedback_0'], 'json', '{0}.json'.format(file_name))

    arr_1 = None
    arr_2 = None
    arr_3 = None

    if input_mrc_file and os.path.splitext(input_mrc_file)[1] in ('.mrc', '.mrcs'):
        try:
            with mrc.open(input_mrc_file) as mrc_file:
                input_data = mrc_file.data
        except ValueError:
            with mrc.open(input_mrc_file, 'r+', permissive=True) as mrc_file:
                mrc_file.header.map = mrc.constants.MAP_ID
            with mrc.open(input_mrc_file) as mrc_file:
                input_data = mrc_file.data
        if len(input_data.shape) == 3:
            input_data = np.sum(input_data, axis=0) / input_data.shape[0]
        input_data = input_data - np.mean(input_data)
        input_data = tu.normalize_image(input_data)

        if input_data.shape[0] > 512:
            idx = 1
            while True:
                if np.max(input_data.shape) < idx * 4096:
                    break
                idx += 1
            original_shape = idx * 4096
            bin_shape = 512
            ratio = original_shape / bin_shape
            pad_x = original_shape - input_data.shape[0]
            pad_y = original_shape - input_data.shape[1]
            shape = (bin_shape, bin_shape)
            input_data = np.pad(input_data, ((0, pad_x), (0, pad_y)), mode='median')
            shape = (bin_shape, bin_shape)
            output_data = tu.rebin(input_data, shape)[:-int(1+pad_x//ratio), :-int(1+pad_y//ratio)]
        else:
            output_data = input_data

        arr_1 = output_data

    if input_ctf_file:
        try:
            with mrc.open(input_ctf_file) as mrc_file:
                input_data = mrc_file.data
        except ValueError:
            with mrc.open(input_ctf_file, 'r+', permissive=True) as mrc_file:
                mrc_file.header.map = mrc.constants.MAP_ID
            with mrc.open(input_ctf_file) as mrc_file:
                input_data = mrc_file.data
        if len(input_data.shape) == 3:
            input_data = np.sum(input_data, axis=0) / input_data.shape[0]
        input_data = input_data - np.mean(input_data)

        if input_data.shape[0] > 512*2:
            shape = (512*2, 512*2)
            output_data = tu.rebin(input_data, shape)
        else:
            output_data = input_data
        arr_2 = output_data

    if input_1d_file:
        plot_data = []
        if 'CTFFIND4' in ctf_name:
            data = np.genfromtxt(input_1d_file)
            x_data = data[0]
            plot_data.append([data[1], 'CTF_no_astig'])
            plot_data.append([data[2], 'CTF'])
            plot_data.append([data[3], 'CTF_fit'])
            plot_data.append([data[4], 'CCC'])
            plot_data.append([data[5], '2sigma'])
        elif 'Gctf' in ctf_name:
            data = np.genfromtxt(input_1d_file, skip_header=1)
            x_data = 1/data[:,0]
            plot_data.append([data[:,1], 'CTF'])
            plot_data.append([data[:,3], 'EPA_BG'])
            plot_data.append([data[:,4], 'CCC'])
        elif 'CTER' in ctf_name:
            data = np.genfromtxt(input_1d_file)
            x_data = data[:,1]
            plot_data.append([data[:,2], 'CTF_no_astig'])
            plot_data.append([data[:,3], 'CTF_no_astig_fit'])
            plot_data.append([data[:,4], 'CTF'])
            plot_data.append([data[:,5], 'CTF_fit'])

        arr_3 = plot_data

    if arr_1 is not None:
        mi.imsave(jpg_file_1, arr_1, cmap='gist_gray')
    if arr_2 is not None:
        mi.imsave(jpg_file_2, arr_2, cmap='gist_gray')
    if arr_3 is not None:
        json_dict = {
            'label_x': 'Spacial frequency 1/px',
            'label_y': 'CTF',
            'is_equal': False,
            'data': []
            }
        for idx, (y_values, label) in enumerate(arr_3):
            json_dict['data'].append({
                'values_x': x_data.tolist(),
                'values_y': y_values.tolist(),
                'is_high_res': False,
                'label_plot': label,
                'marker': '-',
                'color': matplotlib.cm.get_cmap('viridis')(idx / (len(arr_3) - 1)),
                })
        with open(json_file, 'w') as write:
            json.dump(json_dict, write, indent=1)



"""
    TranSPHIRE is supposed to help with the cryo-EM data collection
    Copyright (C) 2017 Markus Stabrin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
from PyQt5.QtWidgets import QHBoxLayout, QVBoxLayout, QWidget, QLabel
from PyQt5.QtGui import QImage
from PyQt5.QtCore import pyqtSlot, pyqtSignal
from . import transphire_utils as tu
from . import logviewer as lv


class StatusContainer(QWidget):
    """
    Container for status widgets

    Inherits:
    QWidget

    Signals:
    sig_refresh_quota - Connected to change the quota text (no object)
    """
    sig_refresh_quota = pyqtSignal()

    def __init__(self, content, content_mount, content_pipeline, mount_worker, process_worker, content_font, parent=None, **kwargs):
        """
        Layout for the status container.

        Arguments:
        content - Content to fill the statuscontainer
        content_mount - Content for the mount points
        content_pipeline - Content for the pipeline settings
        mount_worker - MountWorker object
        process_worker - ProcessWorker object
        parent - Parent widget (default None)

        Returns:
        None
        """
        super(StatusContainer, self).__init__(parent)

        # Layout
        central_layout = QVBoxLayout(self)
        central_layout.setContentsMargins(0, 0, 0, 0)
        central_widget = QWidget(self)
        central_widget.setObjectName('central_black')
        central_layout.addWidget(central_widget)

        layout_v1 = QVBoxLayout(central_widget)

        for entry in content:
            for widget in entry:
                for key in widget:
                    if key == 'Image':
                        image = widget[key][0]

        # Global content
        self.content = {}

        # Add em-transfer quota
        self.content['scratch'] = StatusWidget(name='scratch', default_name='Connected', default_quota='-- / --')
        layout_v1.addWidget(self.content['scratch'])

        # Add em-transfer quota
        self.content['project'] = StatusWidget(name='project', default_name='Connected', default_quota='-- / --')
        layout_v1.addWidget(self.content['project'])

        content_temp = []
        for entry in content_mount:
            my_content = {}
            for widget in entry:
                for key in widget:
                    my_content[key] = widget[key]
            content_temp.append(my_content)

        # Content
        for entry in content_temp:
            key = entry['Mount name'][0]
            if not key:
                continue
            elif key == 'HDD':
                max_iter = 6
            else:
                max_iter = 1
            for i in range(max_iter):
                if max_iter == 1:
                    key_name = key
                    key_device = key.replace(' ', '_')
                else:
                    key_name = '{0} {1}'.format(key, i)
                    key_device = key_name.replace(' ', '_')

                mount_worker.sig_add_save.emit(
                    key_device,
                    entry['SSH address'][0],
                    entry['Quota command'][0],
                    entry['Is df giving the right quota?'][0],
                    entry['Quota / TB'][0]
                    )

                self.content[key_device] = StatusWidget(
                    name=key_name, default_name='Not connected', default_quota='-- / --'
                    )
                self.content[key_device].setVisible(False)
                layout_v1.addWidget(self.content[key_device])

        # Add a visual separator
        layout_v1.addWidget(Separator(typ='horizontal', color='grey', parent=self))

        self.content['Feedbacks'] = StatusWidget(name='Feedbacks', default_name='00|00', default_quota='Not running')
        layout_v1.addWidget(self.content['Feedbacks'])
        # Add process status widgets
        for entry in content_pipeline[0]:
            for key in entry:
                if 'WIDGETS' in key:
                    continue
                basename = key
                name = basename
                self.content[name] = StatusWidget(name=name, default_name='00|00', default_quota='Not running')
                layout_v1.addWidget(self.content[name])

        layout_v1.addWidget(Separator(typ='horizontal', color='grey', parent=self))

        self.log_viewer = lv.LogViewer(
            show_indicators=True,
            file_name='',
            parent=self
            )
        layout_v1.addWidget(self.log_viewer, stretch=1)

        # Add picture
        if image and os.path.exists(image):
            pass
        else:
            image = os.path.join(os.path.dirname(__file__), 'images', 'Transphire.png')
        qimage = QImage(image)

        image_height = float(content_font[0][0]['Font'][0]) * 150 / 10
        try:
            image_width = image_height * qimage.width() / qimage.height()
        except ZeroDivisionError:
            tu.message('Chosen picture: "{0}" - No longer available!'.format(image))
            self.log_viewer.appendPlainText('Chosen picture: "{0}" - No longer available!'.format(image))
        else:
            pic_label = QLabel(self)
            pic_label.setObjectName('picture')
            pic_label.setStyleSheet('border-image: url("{0}")'.format(image))
            pic_label.setMaximumHeight(image_height)
            pic_label.setMinimumHeight(image_height)
            pic_label.setMaximumWidth(image_width)
            pic_label.setMinimumWidth(image_width)

            layout_image = QHBoxLayout()
            layout_image.addStretch(1)
            layout_image.addWidget(pic_label)
            layout_image.addStretch(1)

            layout_v1.addLayout(layout_image)

        # Reset quota warning
        mount_worker.quota_warning = True

        # Events
        mount_worker.sig_success.connect(self._mount_success)
        mount_worker.sig_error.connect(self._mount_error)
        mount_worker.sig_info.connect(self._mount_info)
        mount_worker.sig_quota.connect(self.refresh_quota)

        process_worker.sig_status.connect(self._process_success)
        process_worker.sig_set_project_directory.connect(self.log_viewer.set_project_path)
        process_worker.sig_error.connect(self._process_error)

        self.sig_refresh_quota.connect(mount_worker.refresh_quota)

    @pyqtSlot(str, str, str)
    def _mount_success(self, text, device, color):
        """
        Mount was successfull.

        Arguments:
        text - Text to show
        device - Device name
        color - Color of the text

        Returns:
        None
        """
        self.content[device].sig_change_info_name.emit(text, color)
        if color == 'white':
            mount_type = 'Unmount'
            self.content[device].setVisible(False)
        else:
            mount_type = 'Mount'
            self.content[device].setVisible(True)
        if device != 'scratch' and device != 'project':
            self.log_viewer.appendPlainText('{0} {1} successfull!'.format(device, mount_type))
            #tu.message('{0} Mount/Unmount successfull!'.format(device))
        else:
            pass

    @pyqtSlot(str, str)
    def _mount_error(self, text, device):
        """
        Mount was not successfull

        Arguments:
        text - Text to show
        device - Device name

        Returns:
        None
        """
        if device != 'None':
            self.content[device].sig_change_info_name.emit('Not connected', 'red')
        self.log_viewer.appendPlainText(text)
        #tu.message(text)

    @pyqtSlot(str)
    def _mount_info(self, text):
        """
        Information for the user

        Arguments:
        text - Text to show

        Returns:
        None
        """
        self.log_viewer.appendPlainText(text)
        #tu.message(text)

    @pyqtSlot(str, object, str, str)
    def _process_success(self, text, numbers, device, color):
        """
        Success in a process

        Arguments:
        text - Text to show
        device - Device name
        color - Color of the text

        Returns:
        None
        """
        text = text.split()
        quota_list = []
        for entry in numbers:
            try:
                entry = '{0:.0f}'.format(entry)
            except:
                entry = entry
            quota_list.append(entry)

        self.content[device].sig_change_info_name.emit(' '.join(text), color)
        self.content[device].sig_change_info_quota.emit(' | '.join(quota_list), color)
        self.sig_refresh_quota.emit()

    @pyqtSlot(str)
    def _process_error(self, text):
        """
        Error in a process

        Arguments:
        text - Text to show

        Returns:
        None
        """
        self.log_viewer.appendPlainText(text, indicator='error')
        self.sig_refresh_quota.emit()

    @pyqtSlot(str, str, str)
    def refresh_quota(self, text, device, color):
        """
        Refresh the quota

        Arguments:
        text - Text to show
        device - Device name
        color - Color of the text

        Returns:
        None
        """
        self.content[device].sig_change_info_quota.emit(text, color)

from .statuswidget import StatusWidget
from .separator import Separator
